# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'settingsui.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
from PyQt5 import QtCore, QtWidgets


class Ui_ShortcutsUI(object):

    def setupUi(self, ShortcutsUI):
        ShortcutsUI.setObjectName("ShortcutsUI")
        ShortcutsUI.resize(579, 334)

        self.gridLayout = QtWidgets.QGridLayout(ShortcutsUI)
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("verLayout")

        self.groupBox = QtWidgets.QGroupBox(ShortcutsUI)
        self.groupBox.setObjectName("groupBox")
        self.groupBox.setContentsMargins(11, 11, 11, 11)

        self.vLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.vLayout.setContentsMargins(11, 11, 11, 11)

        self.tableView = QtWidgets.QTableView()
        self.tableView.setContentsMargins(11, 11, 11, 11)

        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 0)
        self.vLayout.addWidget(self.tableView)

        self.closeButton = QtWidgets.QPushButton()
        self.closeButton.setObjectName("closeButton")
        self.closeButton.setMaximumWidth(150)
        self.closeButton.setContentsMargins(11, 11, 11, 11)

        self.gridLayout.addWidget(self.closeButton, 4, 2)
        self.gridLayout.setSpacing(10)

        self.retranslateUi(ShortcutsUI)
        QtCore.QMetaObject.connectSlotsByName(ShortcutsUI)

    def retranslateUi(self, ShortcutsUI):
        _translate = QtCore.QCoreApplication.translate
        ShortcutsUI.setWindowTitle(_translate("ShortcutsUI", "Shortcuts"))
        self.groupBox.setTitle(_translate("ShortcutsUI", "Shortcuts list"))
        self.closeButton.setText(_translate("ShortcutsUI", "Close"))



import os
import string
from datetime import date
from datetime import timedelta
import re

from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5 import QtWidgets

from qtodotxt.lib import tasklib
from qtodotxt.lib.task_htmlizer import TaskHtmlizer

from qtodotxt.lib.tasklib import recursiveMode


class LinkDialog(QtWidgets.QFileDialog):
    def __init__(self, parent, directory):
        QtWidgets.QFileDialog.__init__(self, parent, caption="Select file", directory=directory)
        self.setFileMode(QtWidgets.QFileDialog.AnyFile)
        self.setViewMode(QtWidgets.QFileDialog.Detail)

    @staticmethod
    def getLink(parent, directory):
        dia = LinkDialog(parent, directory=directory)
        if dia.exec_():
            return [uri.toString() for uri in dia.selectedUrls()]


class TasksListController(QtCore.QObject):

    taskModified = QtCore.pyqtSignal(tasklib.Task)
    taskCreated = QtCore.pyqtSignal(tasklib.Task)
    taskArchived = QtCore.pyqtSignal(tasklib.Task)
    taskDeleted = QtCore.pyqtSignal(tasklib.Task)

    def __init__(self, view, mfile):
        QtCore.QObject.__init__(self)
        self.style = ":/white_icons"
        if str(QtCore.QSettings().value("color_schem", "")).find("dark") >= 0:
            self.style = ":/dark_icons"
        self._settings = QtCore.QSettings()
        self.view = view
        self.view.setFileObject(mfile)
        self._task_htmlizer = TaskHtmlizer()
        self._priorities = ["(" + i + ")" for i in string.ascii_uppercase]  # FIXME: why do we have this?
        self.view.taskActivated.connect(self.view.editCurrentTask)
        self.view.currentTaskChanged.connect(self.updateActions)
        self.view.taskDeleted.connect(self.taskDeleted.emit)
        self.view.itemSelectionChanged.connect(self.updateActions)
        self._initCreateTaskAction()
        self._initEditTaskAction()
        self._initCopySelectedTasksAction()
        if int(QtCore.QSettings().value("show_delete", 1)):
            self._initDeleteSelectedTasksAction()
        self._initCompleteSelectedTasksAction()
        self._initDecreasePrioritySelectedTasksAction()
        self._initIncreasePrioritySelectedTasksAction()
        self._initCreateTaskActionOnTemplate()
        self._initAddLinkAction()
        self.view.taskCreated.connect(self._task_created)
        self.view.taskModified.connect(self._task_modified)
        self.disableTaskActions()

    def _task_created(self, task):
        self.view.clearSelection()
        self.view.selectTask(task)
        self.taskCreated.emit(task)

    def _task_modified(self, task):
        self.taskModified.emit(task)

    def _initEditTaskAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self.style + '/resources/TaskEdit.png'), self.tr('&Edit Task'), self)
        action.setShortcuts(['Ctrl+E', 'Enter'])
        action.setDisabled(True)
        action.triggered.connect(self.view.editCurrentTask)
        self.view.addAction(action)
        self.editTaskAction = action

    def _initCreateTaskAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskCreate.png'), self.tr('&Create new task'), self)
        action.setShortcuts(['Insert', 'Ctrl+I', 'Ctrl+N'])
        action.triggered.connect(self.view.createTask)
        self.view.addAction(action)
        self.createTaskAction = action

    def _initCreateTaskActionOnTemplate(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskAddOnTem.png'),
            self.tr('&Create a new task based on current task'), self)
        action.setShortcuts(['Shift+Insert', 'Ctrl+Shift+I'])
        action.triggered.connect(self.createTaskOnTemplate)
        self.view.addAction(action)
        self.createTaskActionOnTemplate = action

    def _initCopySelectedTasksAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskCopy.png'), self.tr('Copy selected tasks'), self)
        action.setShortcuts([QtGui.QKeySequence.Copy])
        action.triggered.connect(self._copySelectedTasks)
        self.view.addAction(action)
        self.copySelectedTasksAction = action

    def _initDeleteSelectedTasksAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskDelete.png'), self.tr('&Delete selected tasks'), self)
        action.setShortcut('Delete')
        action.triggered.connect(self._deleteSelectedTasks)
        self.view.addAction(action)
        self.deleteSelectedTasksAction = action

    def _initCompleteSelectedTasksAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskComplete.png'), self.tr('C&omplete selected tasks'), self)
        action.setShortcuts(['x', 'c'])
        action.triggered.connect(self._completeSelectedTasks)
        self.view.addAction(action)
        self.completeSelectedTasksAction = action

    def _initDecreasePrioritySelectedTasksAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskPriorityDecrease.png'), self.tr('Decrease priority'), self)
        action.setShortcuts(['-', '<'])
        action.triggered.connect(self._decreasePriority)
        self.view.addAction(action)
        self.decreasePrioritySelectedTasksAction = action

    def _initIncreasePrioritySelectedTasksAction(self):
        action = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/TaskPriorityIncrease.png'), self.tr('Increase priority'), self)
        action.setShortcuts(['+', '>'])
        action.triggered.connect(self._increasePriority)
        self.view.addAction(action)
        self.increasePrioritySelectedTasksAction = action

    def _initAddLinkAction(self):
        self.addLinkAction = QtWidgets.QAction(
            QtGui.QIcon(self.style + '/resources/link.png'), self.tr('Add &Link to file'), self)
        self.addLinkAction.setShortcuts(['Ctrl+Shift+L'])
        self.view.addAction(self.addLinkAction)
        self.addLinkAction.triggered.connect(self._addLink)

    def _addLink(self):
        tasks = self.view.getSelectedTasks()
        if tasks:
            uris = LinkDialog.getLink(self.view, directory=".")
            if uris:
                for uri in uris:
                    for task in tasks:
                        task.text = task.text + " " + uri
                        self.taskModified.emit(task)

    def completeTask(self, task):
        if not task.is_complete:
            # Check if task is recurrent and has a due date
            if task.recursion is not None and task.due is not None:
                self._recurTask(task)
            task.setCompleted()  # maybe we should not do that is show_completed is True....
            self.view.selectNext()
        else:
            task.setPending()
        if int(QtCore.QSettings().value("auto_archive", 0)):
            self.taskArchived.emit(task)
        else:
            self.taskModified.emit(task)

    def _recurTask(self, task):
        if task.recursion.interval == 'd':
            if task.recursion.mode == recursiveMode.originalDueDate:
                next_due_date = task.due + timedelta(days=int(task.recursion.increment))
            else:
                next_due_date = date.today() + timedelta(days=int(task.recursion.increment))
        elif task.recursion.interval == 'b':
            if task.recursion.mode == recursiveMode.originalDueDate:
                next_due_date = self._incrWorkDays(task.due, int(task.recursion.increment))
            else:
                next_due_date = self._incrWorkDays(date.today(), int(task.recursion.increment))
        elif task.recursion.interval == 'w':
            if task.recursion.mode == recursiveMode.originalDueDate:
                next_due_date = task.due + timedelta(weeks=int(task.recursion.increment))
            else:
                next_due_date = date.today() + timedelta(weeks=int(task.recursion.increment))
        elif task.recursion.interval == 'm':
            if task.recursion.mode == recursiveMode.originalDueDate:
                next_due_date = task.due + timedelta(weeks=int(task.recursion.increment) * 4)  # 4 weeks in a month
            else:
                next_due_date = date.today() + timedelta(weeks=int(task.recursion.increment) * 4)  # 4 weeks in a month
        elif task.recursion.interval == 'y':
            if task.recursion.mode == recursiveMode.originalDueDate:
                next_due_date = task.due + timedelta(weeks=int(task.recursion.increment) * 52)  # 52 weeks in a year
            else:
                next_due_date = date.today() + timedelta(weeks=int(task.recursion.increment) * 52)  # 52 weeks in a year
        else:
            # Test already made during line parsing - shouldn't be a problem here
            pass
        # Set new due date in old task text
        rec_text = task.updateDateInTask(task.text, next_due_date)
        # create a new task duplicate
        self._createTask(rec_text)
        return

    def _incrWorkDays(self, startDate, daysToIncrement):
        while daysToIncrement > 0:
            if startDate.weekday() == 4:  # Friday
                startDate = startDate + timedelta(days=3)
            elif startDate.weekday() == 5:  # Saturday
                startDate = startDate + timedelta(days=2)
            else:
                startDate = startDate + timedelta(days=1)
            daysToIncrement -= 1
        return startDate

    def _completeSelectedTasks(self):

        tasks = self.view.getSelectedTasks()
        if tasks:
            confirm = int(QtCore.QSettings().value("confirm_complete", 1))
            if not confirm or self._confirmTasksAction(tasks, self.tr('Toggle completeness of')):
                for task in tasks:
                    self.completeTask(task)

    def _deleteSelectedTasks(self):
        tasks = self.view.getSelectedTasks()
        if tasks:
            if self._confirmTasksAction(tasks, self.tr('Delete')):
                for task in tasks:
                    self.view.removeTask(task)
                    self.taskDeleted.emit(task)
                    self.view.selectNext()

    def _confirmTasksAction(self, tasks, messagePrefix):
        if len(tasks) == 1:
            message = self.tr('<b>%s the following task?</b><ul>') % messagePrefix
        else:
            message = self.tr('<b>%s the following tasks?</b><ul>') % messagePrefix
        for task in tasks:
            message += '<li>%s</li>' % self._task_htmlizer.task2html(task)
        message += '</ul>'
        result = QtWidgets.QMessageBox.question(
            self.view,
            self.tr('Confirm'),
            message,
            buttons=QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            defaultButton=QtWidgets.QMessageBox.Yes)
        return result == QtWidgets.QMessageBox.Yes

    def _decreasePriority(self):
        tasks = self.view.getSelectedTasks()
        if tasks:
            for task in tasks:
                task.decreasePriority()
                self.taskModified.emit(task)

    def _increasePriority(self):
        tasks = self.view.getSelectedTasks()
        if tasks:
            for task in tasks:
                task.increasePriority()
                self.taskModified.emit(task)

    def showTasks(self, tasks):
        previouslySelectedTasks = self.view.getSelectedTasks()
        self.view.clear()
        self._sortTasks(tasks)
        for task in tasks:
            self.view.addTask(task)
        self._reselect(previouslySelectedTasks)

    def _reselect(self, tasks):
        for task in tasks:
            self.view.selectTaskByText(task.text)

    def _sortTasks(self, tasks):
        tasks.sort(reverse=True)

    def _removeCreationDate(self, text):
        match = re.match(r'^(\([A-Z]\)\s)?[0-9]{4}\-[0-9]{2}\-[0-9]{2}\s(.*)', text)
        if match:
            if match.group(1):
                text = match.group(1) + match.group(2)
            else:
                text = match.group(2)
        return text

    def _addCreationDate(self, text):
        date_string = date.today().strftime('%Y-%m-%d')
        if text[:3] in self._priorities:
            text = '%s %s %s' % (text[:3], date_string, text[4:])
        else:
            text = '%s %s' % (date_string, text)
        return text

    def _createTask(self, text):
        if int(QtCore.QSettings().value("add_created_date", 0)):
            text = self._removeCreationDate(text)
            text = self._addCreationDate(text)
        task = tasklib.Task(text)
        self.view.addTask(task)
        self._task_created(task)
        return task

    def createTaskOnTemplate(self):
        tasks = self.view.getSelectedTasks()
        if len(tasks) != 1:
            return
        task = tasks[0]
        return self.view.createTask(task)

    def _copySelectedTasks(self):
        tasks = self.view.getSelectedTasks()
        if tasks:
            text = "".join(str(task) + os.linesep for task in tasks)
            app = QtWidgets.QApplication.instance()
            app.clipboard().setText(text)

    def updateActions(self):
        tasks = self.view.selectedItems()
        if tasks:
            self.enableTaskActions()
            if len(tasks) > 1:
                self.editTaskAction.setEnabled(False)
                self.addLinkAction.setEnabled(False)
                self.createTaskActionOnTemplate.setEnabled(False)
        else:
            self.disableTaskActions()

    def enableTaskActions(self):
        self.editTaskAction.setEnabled(True)
        self.addLinkAction.setEnabled(True)
        self.createTaskActionOnTemplate.setEnabled(True)
        self.deleteSelectedTasksAction.setEnabled(True)
        self.completeSelectedTasksAction.setEnabled(True)
        self.copySelectedTasksAction.setEnabled(True)
        self.increasePrioritySelectedTasksAction.setEnabled(True)
        self.decreasePrioritySelectedTasksAction.setEnabled(True)

    def disableTaskActions(self):
        self.editTaskAction.setEnabled(False)
        self.addLinkAction.setEnabled(False)
        self.createTaskActionOnTemplate.setEnabled(False)
        self.deleteSelectedTasksAction.setEnabled(False)
        self.completeSelectedTasksAction.setEnabled(False)
        self.copySelectedTasksAction.setEnabled(False)
        self.increasePrioritySelectedTasksAction.setEnabled(False)
        self.decreasePrioritySelectedTasksAction.setEnabled(False)



#!/usr/bin/env python

from os import path
from subprocess import call
from sys import platform

OUTPUT_DIR = path.join('.testOutput', 'html')

if platform == 'win32':
    from os import startfile

    def openfile():
        startfile(path.join(OUTPUT_DIR, 'index.html'))
else:
    def openfile():
        call(['xdg-open', path.join(OUTPUT_DIR, 'index.html')])

call(['coverage', 'run', 'run-tests.py'])
call(['coverage', 'html', '-d', OUTPUT_DIR])
openfile()



from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5 import QtWidgets
import os

from qtodotxt.ui.resource_manager import getResourcePath
from qtodotxt.ui.views.filters_tree_view import FiltersTreeView
from qtodotxt.ui.views.tasks_view import TasksView


class MainView(QtWidgets.QMainWindow):

    closeEventSignal = QtCore.pyqtSignal(QtGui.QCloseEvent)

    def __init__(self, parent=None):
        super(MainView, self).__init__(parent)
        self.style = ":/white_icons"
        if str(QtCore.QSettings().value("color_schem", "")).find("dark") >= 0:
            self.style = ":/dark_icons"
        self._initUI()

    def show(self):
        super(MainView, self).show()

    def hide(self):
        super(MainView, self).hide()

    def _initUI(self):

        self.splitter = QtWidgets.QSplitter()
        self.splitter.setHandleWidth(1)

        self.filters_tree_view = FiltersTreeView(self.splitter)
        self.tasks_view = TasksView(self.splitter)

        colorSchemName = QtCore.QSettings().value("color_schem", "")
        tFile = os.path.join("css", "{}.css".format(colorSchemName))
        cssPath = getResourcePath(tFile)
        if os.path.isfile(cssPath):
            css = open(cssPath, 'r', encoding='utf-8').read()
            self.setStyleSheet(css)

        self.setCentralWidget(self.splitter)

        self.resize(800, 400)
        self.splitter.setSizes([250, 550])
        self.setWindowIcon(QtGui.QIcon(self.style + '/resources/qtodotxt.png'))

        self.setTabOrder(self.tasks_view.tasks_search_view, self.tasks_view.tasks_list_view)
        self.setTabOrder(self.tasks_view.tasks_list_view, self.filters_tree_view)

    def closeEvent(self, closeEvent):
        super(MainView, self).closeEvent(closeEvent)
        self.closeEventSignal.emit(closeEvent)



from PyQt5 import QtCore, QtGui, QtWidgets


class TasksSearchView(QtWidgets.QLineEdit):

    searchTextChanged = QtCore.pyqtSignal(str)

    def __init__(self, searchIcon, clearIcon, parent=None):
        QtWidgets.QLineEdit.__init__(self, parent)

        self._searchText = ""

        self.clearButton = QtWidgets.QToolButton(self)
        self.clearButton.setIcon(clearIcon)
        self.clearButton.setCursor(QtCore.Qt.ArrowCursor)
        self.clearButton.setStyleSheet("QToolButton { border: none; padding: 0px; }")
        self.clearButton.hide()
        self.clearButton.clicked.connect(self.clear)
        self.textChanged.connect(self.updateSearchText)

        self.searchButton = QtWidgets.QToolButton(self)
        self.searchButton.setIcon(searchIcon)
        self.searchButton.setStyleSheet("QToolButton { border: none; padding: 0px; }")

        frameWidth = self.style().pixelMetric(QtWidgets.QStyle.PM_DefaultFrameWidth)
        buttonWidth = self.clearButton.sizeHint().width()
        self.setStyleSheet("QLineEdit { padding-left: %spx; padding-right: %spx; } " % (
            self.searchButton.sizeHint().width() + frameWidth + 1, buttonWidth + frameWidth + 1))
        msz = self.minimumSizeHint()
        self.setMinimumSize(max(msz.width(),
                                self.searchButton.sizeHint().width() + buttonWidth + frameWidth * 2 + 2),
                            max(msz.height(), self.clearButton.sizeHint().height() + frameWidth * 2 + 2))
        self.setPlaceholderText("Search")

        focusShortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+F"), self)
        focusShortcut.activated.connect(self.setFocus)

    def resizeEvent(self, event):
        sz = self.clearButton.sizeHint()
        frameWidth = self.style().pixelMetric(QtWidgets.QStyle.PM_DefaultFrameWidth)
        self.clearButton.move(self.rect().right() - frameWidth - sz.width(),
                              (self.rect().bottom() + 1 - sz.height()) / 2)
        self.searchButton.move(self.rect().left() + 1, (self.rect().bottom() + 1 - sz.height()) / 2)

    def getSearchText(self):
        return self._searchText

    def updateSearchText(self, searchText):
        self._searchText = searchText
        self.searchTextChanged.emit(searchText)
        self.updateCloseButton(bool(searchText))

    def updateCloseButton(self, visible):
            self.clearButton.setVisible(visible)



#!/usr/bin/env python3

import doctest
import os
from os import path
import sys

try:
    import PyQt5  # noqa
except ImportError:
    print("Couldn't import PyQt5. Aborting. These are the PYTHONPATHs:")
    print(sys.path)
    sys.exit(1)


testsdir = path.abspath(path.dirname(__file__))
root = path.abspath(path.join(testsdir, '..', '..'))
sys.path.insert(0, root)


def run_doctests(testsdir):
    exit_code = 0
    for filename in os.listdir(testsdir):
        fullname = path.join(testsdir, filename)
        if filename.endswith('.doctest'):
            print("- Running", fullname)
            result = doctest.testfile(fullname, module_relative=False)
            print("  => ran {0} results, {1} failed".format(result.attempted, result.failed))
            exit_code += result.failed
        elif path.isdir(fullname):
            exit_code += run_doctests(fullname)

    return exit_code


if __name__ == "__main__":
    exit_code = 0

    print("Running Doctests...")
    exit_code = run_doctests(testsdir)
    sys.exit(exit_code)



import sys

from PyQt5 import QtCore, QtGui, QtWidgets

from qtodotxt.ui.dialogs.settingsui import Ui_SettingsUI


class Settings(QtWidgets.QDialog):

    def __init__(self, maincontroller):
        QtWidgets.QDialog.__init__(self, maincontroller.view)
        self.maincontroller = maincontroller
        self.ui = Ui_SettingsUI()
        self.ui.setupUi(self)
        self.settings = QtCore.QSettings()
        self.setWindowFlags(
                QtCore.Qt.Dialog |
                QtCore.Qt.MSWindowsFixedSizeDialogHint |
                QtCore.Qt.WindowStaysOnBottomHint |
                QtCore.Qt.WindowSystemMenuHint |
                QtCore.Qt.WindowTitleHint |
                QtCore.Qt.WindowCloseButtonHint
                )

        self.load_settings()
        self.connect_all()

    def load_settings(self):
        self._int_settings_to_cb("auto_save", self.ui.autoSaveCheckBox)
        self._int_settings_to_cb("singleton", self.ui.singletonCheckBox, 0)
        self._int_settings_to_cb("auto_archive", self.ui.autoArchiveCheckBox, 0)
        self._int_settings_to_cb("add_created_date", self.ui.addCreatedDateCheckBox, 0)
        self._int_settings_to_cb("confirm_complete", self.ui.confirmCompletionCheckBox)
        self._int_settings_to_cb("show_delete", self.ui.deleteActionCheckBox, 1)
        priority = self.settings.value("lowest_priority", "D")
        self.ui.lowestPriorityLineEdit.setText(priority)
        self._int_settings_to_cb("enable_tray", self.ui.trayCheckBox, 0)
        self._int_settings_to_cb("hide_to_tray", self.ui.hideToTrayCheckBox, 0)
        self._int_settings_to_cb("hide_on_startup", self.ui.hideOnStartupCheckBox, 0)
        self._int_settings_to_cb("close_to_tray", self.ui.closeToTrayCheckBox, 0)

        val = int(self.settings.value("enable_tray", 0))
        self.ui.hideToTrayCheckBox.setEnabled(val)
        self.ui.hideOnStartupCheckBox.setEnabled(val)
        self.ui.closeToTrayCheckBox.setEnabled(val)

        val = self.settings.value("color_schem", "")
        index = self.ui.colorSchemComboBox.findText(val, QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.ui.colorSchemComboBox.setCurrentIndex(index)

    def _int_settings_to_cb(self, name, checkBox, default=1):
        val = int(self.settings.value(name, default))
        if val:
            checkBox.setCheckState(QtCore.Qt.Checked)
        else:
            checkBox.setCheckState(QtCore.Qt.Unchecked)

    def connect_all(self):
        self.ui.closeButton.clicked.connect(self.close)
        self.ui.singletonCheckBox.stateChanged.connect(self.setSingletonCheckBox)
        self.ui.autoSaveCheckBox.stateChanged.connect(self.setAutoSave)
        self.ui.autoArchiveCheckBox.stateChanged.connect(self.setAutoArchive)
        self.ui.addCreatedDateCheckBox.stateChanged.connect(self.setAddCreatedDate)
        self.ui.confirmCompletionCheckBox.stateChanged.connect(self.setConfirmCompletion)
        self.ui.deleteActionCheckBox.stateChanged.connect(self.setDeleteAction)
        self.ui.lowestPriorityLineEdit.textChanged.connect(self.setLowestPriority)
        self.ui.trayCheckBox.stateChanged.connect(self.enableTray)
        self.ui.hideToTrayCheckBox.stateChanged.connect(self.setHideToTray)
        self.ui.hideOnStartupCheckBox.stateChanged.connect(self.setHideOnStartup)
        self.ui.closeToTrayCheckBox.stateChanged.connect(self.setCloseToTray)
        self.ui.colorSchemComboBox.currentIndexChanged.connect(self.setColorSchemCombo)

    def _save_int_cb(self, name, val):
        if val == 0:
            self.settings.setValue(name, 0)
        else:
            self.settings.setValue(name, 1)

    def setSingletonCheckBox(self, val):
        self._save_int_cb("singleton", val)

    def setAutoSave(self, val):
        self._save_int_cb("auto_save", val)

    def setDeleteAction(self, val):
        self._save_int_cb("show_delete", val)

    def setAutoArchive(self, val):
        self._save_int_cb("auto_archive", val)

    def setAddCreatedDate(self, val):
        self._save_int_cb("add_created_date", val)

    def setConfirmCompletion(self, val):
        self._save_int_cb("confirm_complete", val)

    def setLowestPriority(self, text):
        self.settings.setValue("lowest_priority", text)

    def enableTray(self, val):
        self._save_int_cb("enable_tray", val)
        self.ui.hideToTrayCheckBox.setEnabled(val)
        self.ui.hideOnStartupCheckBox.setEnabled(val)
        self.ui.closeToTrayCheckBox.setEnabled(val)

    def setHideToTray(self, val):
        self._save_int_cb("hide_to_tray", val)

    def setHideOnStartup(self, val):
        self._save_int_cb("hide_on_startup", val)

    def setCloseToTray(self, val):
        self._save_int_cb("close_to_tray", val)

    def closeEvent(self, event):
        self.deleteLater()
        self.maincontroller.view.show()

    def setColorSchemCombo(self, val):
        name = self.ui.colorSchemComboBox.itemText(val)
        self.settings.setValue("color_schem", name)


if __name__ == "__main__":
    QtCore.QCoreApplication.setOrganizationName("QTodoTxt")
    QtCore.QCoreApplication.setApplicationName("QTodoTxt")
    app = QtGui.QApplication(sys.argv)
    s = Settings(None)
    s.show()
    sys.exit(app.exec_())



import logging
import os
from PyQt5 import QtCore
from qtodotxt.lib.filters import DueTodayFilter, DueTomorrowFilter, DueThisWeekFilter, DueThisMonthFilter, \
    DueOverdueFilter
from qtodotxt.lib.tasklib import Task
from sys import version
import time

logger = logging.getLogger(__name__)

PYTHON_VERSION = version[:3]

if PYTHON_VERSION < '3.3':
    FileNotFoundError = OSError


class Error(Exception):
    pass


class ErrorLoadingFile(Error):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class ErrorSavingFile(Error):
    def __init__(self, message, innerException=None):
        self.message = message
        self.innerException = innerException

    def __str__(self):
        lines = [repr(self.message)]
        if self.innerException:
            lines.append(repr(self.innerException))
        return '\n'.join(lines)


class File(object):
    def __init__(self):
        self.NEWLINE = '\n'
        self.tasks = []
        self.filename = ''

    def __str__(self):
        return "File(filename:{}, tasks:{})".format(self.filename, self.tasks)
    __repr__ = __str__

    def load(self, filename):

        try:
            with open(filename, 'rt', encoding='utf-8') as fd:
                lines = fd.readlines()
        except FileNotFoundError:
            raise ErrorLoadingFile("Trying to load a non-existing file: '{}".format(filename))
        except IOError as ex:                # deprecated since Python 3.3, it would be OSError for =>3.3-support
            raise ErrorLoadingFile(str(ex))
        self.filename = filename
        self._createTasksFromLines(lines)

    def _createTasksFromLines(self, lines):
        self.tasks = []
        for line in lines:
            task_text = line.strip()
            if task_text:
                task = Task(task_text)
                self.tasks.append(task)

    def save(self, filename=''):
        logger.debug('File.save called with filename="{}"'.format(filename))
        if not filename and not self.filename:
            self.filename = self._createNewFilename()
        elif filename:
            self.filename = filename
        self.tasks.sort(reverse=True)
        self._saveTasks()

    @staticmethod
    def _createNewFilename():
        newFileName = os.path.expanduser('~/todo.txt')
        if not os.path.isfile(newFileName):
            return newFileName
        for counter in range(0, 100):
            newFileName = os.path.expanduser('~/todo.{}.txt.'.format(counter))
            if not os.path.isfile(newFileName):
                return newFileName
        return os.path.expanduser('~/todo.0.txt')

    def _saveTasks(self):
        try:
            with open(self.filename, 'wt', encoding='utf-8') as fd:
                fd.writelines([(task.text + self.NEWLINE) for task in self.tasks])
            logger.debug('{} was saved to disk.'.format(self.filename))
        except IOError as e:
            raise ErrorSavingFile("Error saving to file '{}'".format(self.filename), e)

    def saveDoneTask(self, task):
        doneFilename = os.path.join(os.path.dirname(self.filename), 'done.txt')
        try:
            with open(doneFilename, 'at', encoding='utf-8') as fd:
                fd.write(task.text + self.NEWLINE)
            logger.debug('"{}" was appended to "{}"'.format(task.text, doneFilename))
        except IOError as e:
            raise ErrorSavingFile("Error saving to file '%s'" % doneFilename, e)

    def getAllContexts(self, return_completed=False):
        contexts = dict()
        for task in self.tasks:
            if return_completed or not task.is_complete:
                for context in task.contexts:
                    count = 0 if task.is_complete else 1
                    if context in contexts:
                        contexts[context] += count
                    else:
                        contexts[context] = count
        return contexts

    def getAllProjects(self, return_completed=False):
        projects = dict()
        for task in self.tasks:
            if return_completed or not task.is_complete:
                for project in task.projects:
                    count = 0 if task.is_complete else 1
                    if project in projects:
                        projects[project] += count
                    else:
                        projects[project] = count
        return projects

    def getAllPriorities(self, return_completed=False):
        priorities = {}
        for task in self.tasks:
            if return_completed or not task.is_complete:
                if task.priority in priorities:
                    priorities[task.priority] += 1
                elif task.priority != "":
                    priorities[task.priority] = 1

        return priorities

    def getAllDueRanges(self, return_completed=False):
        dueRanges = dict()
        # This determines the sorting of the ranges in the tree view. Lowest value first.
        rangeSorting = {'Today': 20,
                        'Tomorrow': 30,
                        'This week': 40,
                        'This month': 50,
                        'Overdue': 10}

        for task in self.tasks:
            if not return_completed and task.is_complete:
                continue

            count = 0 if task.is_complete else 1
            if DueTodayFilter('Today').isMatch(task):
                if not ('Today' in dueRanges):
                    dueRanges['Today'] = count
                else:
                    dueRanges['Today'] += count

            if DueTomorrowFilter('Tomorrow').isMatch(task):
                if not ('Tomorrow' in dueRanges):
                    dueRanges['Tomorrow'] = count
                else:
                    dueRanges['Tomorrow'] += count

            if DueThisWeekFilter('This week').isMatch(task):
                if not ('This week' in dueRanges):
                    dueRanges['This week'] = count
                else:
                    dueRanges['This week'] += count

            if DueThisMonthFilter('This month').isMatch(task):
                if not ('This month' in dueRanges):
                    dueRanges['This month'] = count
                else:
                    dueRanges['This month'] += count

            if DueOverdueFilter('Overdue').isMatch(task):
                if not ('Overdue' in dueRanges):
                    dueRanges['Overdue'] = count
                else:
                    dueRanges['Overdue'] += count

        return dueRanges, rangeSorting

    def getTasksCounters(self):
        counters = dict({'Pending': 0,
                         'Uncategorized': 0,
                         'Contexts': 0,
                         'Projects': 0,
                         'Complete': 0,
                         'Priority': 0,
                         'DueCompl': 0,
                         'ProjCompl': 0,
                         'ContCompl': 0,
                         'UncatCompl': 0,
                         'PrioCompl': 0,
                         'Due': 0})
        for task in self.tasks:
            nbProjects = len(task.projects)
            nbContexts = len(task.contexts)
            if not task.is_complete:
                counters['Pending'] += 1
                if nbProjects > 0:
                    counters['Projects'] += 1
                if nbContexts > 0:
                    counters['Contexts'] += 1
                if nbContexts == 0 and nbProjects == 0:
                    counters['Uncategorized'] += 1
                if task.due:
                    counters['Due'] += 1
                if task.priority != "":
                    counters['Priority'] += 1
            else:
                counters['Complete'] += 1
                if nbProjects > 0:
                    counters['ProjCompl'] += 1
                if nbContexts > 0:
                    counters['ContCompl'] += 1
                if nbContexts == 0 and nbProjects == 0:
                    counters['UncatCompl'] += 1
                if task.due:
                    counters['DueCompl'] += 1
                if task.priority != "":
                    counters['PrioCompl'] += 1
        return counters


class FileObserver(QtCore.QFileSystemWatcher):

    fileChangetSig = QtCore.pyqtSignal(str)
    dirChangetSig = QtCore.pyqtSignal(str)

    def __init__(self, parent, file):
        logger.debug('Setting up FileObserver instance.')
        super().__init__(parent)
        self._file = file
        self.fileChanged.connect(self.fileChangedHandler)
        self.directoryChanged.connect(self.dirChangedHandler)

    @QtCore.pyqtSlot(str)
    def fileChangedHandler(self, path):
        logger.debug('Detected change on {}\nremoving it from watchlist'.format(path))
        self.removePath(path)
        debug_counter = 0
        if path == self._file.filename:
            max_time = time.time() + 1
            while time.time() < max_time:
                try:
                    self.fileChangetSig.emit(path)
                except ErrorLoadingFile:
                    time.sleep(0.01)
                    debug_counter += 1
                else:
                    logger.debug('It took {} additional attempts until the file could be read.'.format(debug_counter))
                    break

    @QtCore.pyqtSlot(str)
    def dirChangedHandler(self, path):
        self.dirChangetSig.emit(path)

    def clear(self):
        if self.files():
            logger.debug('Clearing watchlist.')
            self.removePaths(self.files())



from PyQt5 import QtCore, QtWidgets, QtGui
from qtodotxt.ui.views.tasks_search_view import TasksSearchView
from qtodotxt.ui.views.tasks_list_view import TasksListView


class TasksView(QtWidgets.QWidget):

    def __init__(self, parent):
        QtWidgets.QWidget.__init__(self, parent)
        self.style = ":/white_icons"
        if str(QtCore.QSettings().value("color_schem", "")).find("dark") >= 0:
            self.style = ":/dark_icons"
        self._initUI()

    def _initUI(self):
        layout = QtWidgets.QVBoxLayout(self)
        layout.setSpacing(10)

        self.tasks_search_view = TasksSearchView(QtGui.QIcon(self.style + "/resources/ActionSearch.png"),
                                                 QtGui.QIcon(self.style + "/resources/ActionClear.png"), self)
        self.tasks_list_view = TasksListView(self)
        offset = QtCore.QSettings().value("ui_margin_offset", -4)
        self.setContentsMargins(2 * offset, offset, offset, offset)
        layout.addWidget(self.tasks_search_view)
        layout.addWidget(self.tasks_list_view)
        self.setLayout(layout)




import unittest
from datetime import date

from qtodotxt.ui.dialogs.taskeditor_lineedit import end_of_month,  end_of_year, end_of_next_month
from qtodotxt.ui.dialogs.taskeditor_lineedit import end_of_week, end_of_next_week


class TestDates(unittest.TestCase):

    def test_dates(self):
        self.assertEqual("2016-03-06", end_of_week(date(year=2016, month=3, day=5)))
        self.assertEqual("2016-03-06", end_of_week(date(year=2016, month=3, day=1)))
        self.assertEqual("2016-01-03", end_of_week(date(year=2015, month=12, day=31)))
        self.assertEqual("2016-03-13", end_of_next_week(date(year=2016, month=3, day=1)))

        self.assertEqual("2016-02-29", end_of_month(date(year=2016, month=2, day=5)))
        self.assertEqual("2016-02-29", end_of_next_month(date(year=2016, month=1, day=5)))
        self.assertEqual("2015-12-31", end_of_month(date(year=2015, month=12, day=31)))
        self.assertEqual("2016-01-31", end_of_next_month(date(year=2015, month=12, day=31)))

        self.assertEqual("2016-12-31", end_of_month(date(year=2016, month=12, day=5)))
        self.assertEqual("2017-01-31", end_of_next_month(date(year=2016, month=12, day=5)))

        self.assertEqual("2016-12-31", end_of_year(date(year=2016, month=1, day=5)))



import logging
import os
import sys
import time

from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets

from qtodotxt.lib import tasklib
from qtodotxt.lib.file import ErrorLoadingFile, File, FileObserver

from qtodotxt.ui.controllers.tasks_list_controller import TasksListController
from qtodotxt.ui.controllers.filters_tree_controller import FiltersTreeController
from qtodotxt.lib.filters import SimpleTextFilter, FutureFilter, IncompleteTasksFilter, CompleteTasksFilter
from qtodotxt.ui.controllers.menu_controller import MenuController


logger = logging.getLogger(__name__)

FILENAME_FILTERS = ';;'.join([
    'Text Files (*.txt)',
    'All Files (*.*)'])


class MainController(QtCore.QObject):

    _show_toolbar = QtCore.pyqtSignal(int)

    def __init__(self, view, dialogs, args):
        super(MainController, self).__init__()
        self._args = args
        self.view = view

        # use object variable for setting only used in this class
        # others are accessed through QSettings
        self._settings = QtCore.QSettings()

        self._show_completed = True
        self._dialogs = dialogs
        self._file = File()
        self._fileObserver = FileObserver(self, self._file)
        self._initControllers()
        self._is_modified = False
        self._setIsModified(False)
        self._fileObserver.fileChangetSig.connect(self.openFileByName)
        self.view.closeEventSignal.connect(self.view_onCloseEvent)
        filters = self._settings.value("current_filters", ["All"])
        self._filters_tree_controller.view.setSelectedFiltersByNames(filters)
        self.hasTrayIcon = False
        self._menu_controller.updateRecentFileActions()

    def auto_save(self):
        if int(self._settings.value("auto_save", 1)):
            self.save()

    def _initControllers(self):
        self._initFiltersTree()
        self._initTasksList()
        self._initContextualMenu()
        self._initActions()
        self._initMenuBar()
        self._initToolBar()
        self._initSearchText()

    def _initMenuBar(self):
        menu = self.view.menuBar()
        self._menu_controller = MenuController(self, menu)

    def _initActions(self):
        self.filterViewAction = QtWidgets.QAction(QtGui.QIcon(self.view.style + '/resources/sidepane.png'),
                                                  self.tr('Show &Filters'), self)
        self.filterViewAction.setCheckable(True)
        self.filterViewAction.setShortcuts(['Ctrl+Shift+F'])
        self.filterViewAction.triggered.connect(self._toggleFilterView)

        self.showFutureAction = QtWidgets.QAction(QtGui.QIcon(self.view.style + '/resources/future.png'),
                                                  self.tr('Show future &Tasks'), self)
        self.showFutureAction.setCheckable(True)
        self.showFutureAction.setShortcuts(['Ctrl+Shift+T'])
        self.showFutureAction.triggered.connect(self._toggleShowFuture)

        self.showCompletedAction = QtWidgets.QAction(QtGui.QIcon(self.view.style + '/resources/show_completed.png'),
                                                     self.tr('Show &Completed tasks'), self)
        self.showCompletedAction.setCheckable(True)
        self.showCompletedAction.setShortcuts(['Ctrl+Shift+C'])
        self.showCompletedAction.triggered.connect(self._toggleShowCompleted)

        self.archiveAction = QtWidgets.QAction(QtGui.QIcon(self.view.style + '/resources/archive.png'),
                                               self.tr('&Archive completed tasks'), self)
        self.archiveAction.setShortcuts(['Ctrl+Shift+A'])
        self.archiveAction.triggered.connect(self._archive_all_done_tasks)

        self.showToolBarAction = QtWidgets.QAction(self.tr('Show tool&Bar'), self)
        self.showToolBarAction.setCheckable(True)
        self.showToolBarAction.setShortcuts(['Ctrl+Shift+B'])
        self.showToolBarAction.triggered.connect(self._toggleShowToolBar)

        self.showSearchAction = QtWidgets.QAction(QtGui.QIcon(self.view.style + '/resources/ActionSearch.png'),
                                                  self.tr('Show search bar'), self)
        self.showSearchAction.setCheckable(True)
        self.showSearchAction.setShortcuts(['Ctrl+F'])
        self.showSearchAction.triggered.connect(self._toggleShowSearch)

    def _initToolBar(self):
        toolbar = self.view.addToolBar("Main Toolbar")
        toolbar.setObjectName("mainToolbar")

        toolbar.addAction(self.filterViewAction)
        toolbar.addAction(self.showFutureAction)
        toolbar.addAction(self.showCompletedAction)
        toolbar.addAction(self.showSearchAction)

        toolbar.addSeparator()

        toolbar.addAction(self._menu_controller.openAction)
        toolbar.addAction(self._menu_controller.saveAction)
        toolbar.addSeparator()
        toolbar.addAction(self._tasks_list_controller.createTaskAction)
        toolbar.addAction(self._tasks_list_controller.createTaskActionOnTemplate)
        toolbar.addAction(self._tasks_list_controller.editTaskAction)
        toolbar.addAction(self._tasks_list_controller.copySelectedTasksAction)
        toolbar.addSeparator()
        toolbar.addAction(self._tasks_list_controller.completeSelectedTasksAction)
        if int(self._settings.value("show_delete", 0)):
            toolbar.addAction(self._tasks_list_controller.deleteSelectedTasksAction)
        toolbar.addSeparator()
        toolbar.addAction(self._tasks_list_controller.increasePrioritySelectedTasksAction)
        toolbar.addAction(self._tasks_list_controller.decreasePrioritySelectedTasksAction)
        toolbar.addSeparator()
        toolbar.addAction(self.archiveAction)

        toolbar.addSeparator()
        toolbar.addAction(self._tasks_list_controller.addLinkAction)

        self._show_toolbar.connect(toolbar.setVisible)

    def _toggleShowToolBar(self):
        if self.showToolBarAction.isChecked():
            self._settings.setValue("show_toolbar", 1)
            self._toolbar_visibility_changed(1)
        else:
            self._settings.setValue("show_toolbar", 0)
            self._toolbar_visibility_changed(0)

    def _toggleShowSearch(self):
        if self.showSearchAction.isChecked():
            self._settings.setValue("show_search", 1)
            self.view.tasks_view.tasks_search_view.setVisible(True)
        else:
            self._settings.setValue("show_search", 0)
            self.view.tasks_view.tasks_search_view.setVisible(False)
            self.view.tasks_view.tasks_search_view.setText("")

    def _toggleShowCompleted(self):
        if self.showCompletedAction.isChecked():
            self._settings.setValue("show_completed_tasks", 1)
            self._show_completed = True
            self.updateFilters()
        else:
            self._settings.setValue("show_completed_tasks", 0)
            self._show_completed = False
            self.updateFilters()
        self._filters_tree_controller.showFilters(self._file, self._show_completed)

    def _toggleShowFuture(self):
        if self.showFutureAction.isChecked():
            self._settings.setValue("show_future_tasks", 1)
            self.updateFilters()
        else:
            self._settings.setValue("show_future_tasks", 0)
            self.updateFilters()

    def _restoreShowFuture(self):
        val = int(self._settings.value("show_future_tasks", 1))
        if val:
            self.showFutureAction.setChecked(True)
            self._toggleShowFuture()
        else:
            self.showFutureAction.setChecked(False)
            self._toggleShowFuture()

    def _toggleFilterView(self):
        if self.filterViewAction.isChecked():
            self._settings.setValue("show_filter_tree", 1)
            self._filters_tree_controller.view.show()
        else:
            self._settings.setValue("splitter_pos", self.view.centralWidget().sizes())
            self._settings.setValue("show_filter_tree", 0)
            self._filters_tree_controller.view.hide()

    def _restoreFilterView(self):
        val = int(self._settings.value("show_filter_tree", 1))
        if val:
            self.filterViewAction.setChecked(True)
            self._toggleFilterView()
        else:
            self.filterViewAction.setChecked(False)
            self._toggleFilterView()

    def _toolbar_visibility_changed(self, val):
        self._show_toolbar.emit(val)

    def exit(self):
        self.view.close()
        sys.exit()

    def getView(self):
        return self.view

    def show(self):
        self._updateView()
        self.view.show()
        self._updateTitle()

        if self._args.file:
            filename = self._args.file
        else:
            filename = self._settings.value("last_open_file")

        if filename:
            try:
                self.openFileByName(filename)
            except ErrorLoadingFile as ex:
                self._dialogs.showError(str(ex))

        if self._args.quickadd:
            self._tasks_list_controller.createTask()
            self.save()
            self.exit()

    def _initFiltersTree(self):
        controller = self._filters_tree_controller = \
            FiltersTreeController(self.view.filters_tree_view)
        controller.filterSelectionChanged.connect(
            self._onFilterSelectionChanged)

    def _onFilterSelectionChanged(self, filters):
        self._applyFilters(filters=filters)

    def _applyFilters(self, filters=None, searchText=None):
        # First we filter with filters tree
        if filters is None:
            filters = self._filters_tree_controller.view.getSelectedFilters()
        tasks = tasklib.filterTasks(filters, self._file.tasks)
        # Then with our search text
        if searchText is None:
            searchText = self.view.tasks_view.tasks_search_view.getSearchText()
        tasks = tasklib.filterTasks([SimpleTextFilter(searchText)], tasks)
        # with future filter if needed
        if not self.showFutureAction.isChecked():
            tasks = tasklib.filterTasks([FutureFilter()], tasks)
        # with complete filter if needed
        if not CompleteTasksFilter() in filters and not self.showCompletedAction.isChecked():
            tasks = tasklib.filterTasks([IncompleteTasksFilter()], tasks)
        self._tasks_list_controller.showTasks(tasks)

    def _initSearchText(self):
        self.view.tasks_view.tasks_search_view.searchTextChanged.connect(
            self._onSearchTextChanged)

    def _onSearchTextChanged(self, searchText):
        self._applyFilters(searchText=searchText)

    def _initTasksList(self):
        controller = self._tasks_list_controller = \
            TasksListController(self.view.tasks_view.tasks_list_view, self._file)

        controller.taskCreated.connect(self._tasks_list_taskCreated)
        controller.taskModified.connect(self._tasks_list_taskModified)
        controller.taskDeleted.connect(self._tasks_list_taskDeleted)
        controller.taskArchived.connect(self._tasks_list_taskArchived)

    def _initContextualMenu(self):

        # Context menu
        # controller.view.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self._tasks_list_controller.view.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self._tasks_list_controller.view.customContextMenuRequested.connect(self.showContextMenu)
        self._contextMenu = QtWidgets.QMenu(self.view)
        self._contextMenu.addAction(self._tasks_list_controller.createTaskAction)
        self._contextMenu.addAction(self._tasks_list_controller.createTaskActionOnTemplate)
        self._contextMenu.addAction(self._tasks_list_controller.editTaskAction)
        self._contextMenu.addAction(self._tasks_list_controller.copySelectedTasksAction)
        self._contextMenu.addAction(self._tasks_list_controller.addLinkAction)
        self._contextMenu.addSeparator()
        self._contextMenu.addAction(self._tasks_list_controller.completeSelectedTasksAction)
        if int(self._settings.value("show_delete", 1)):
            self._contextMenu.addAction(self._tasks_list_controller.deleteSelectedTasksAction)
        self._contextMenu.addSeparator()
        self._contextMenu.addAction(self._tasks_list_controller.increasePrioritySelectedTasksAction)
        self._contextMenu.addAction(self._tasks_list_controller.decreasePrioritySelectedTasksAction)

    def showContextMenu(self, position):
        tasks = self._tasks_list_controller.view.getSelectedTasks()
        if tasks:
            self._contextMenu.exec_(self._tasks_list_controller.view.mapToGlobal(position))

    def _tasks_list_taskDeleted(self, task):
        self._file.tasks.remove(task)
        self._onFileUpdated()

    def _tasks_list_taskCreated(self, task):
        self._file.tasks.append(task)
        self._onFileUpdated()

    def _tasks_list_taskModified(self):
        self._onFileUpdated()

    def _tasks_list_taskArchived(self, task):
        self._file.saveDoneTask(task)
        self._file.tasks.remove(task)
        self._onFileUpdated()

    def _archive_all_done_tasks(self):
        done = [task for task in self._file.tasks if task.is_complete]
        for task in done:
            self._file.saveDoneTask(task)
            self._file.tasks.remove(task)
        self._onFileUpdated()

    def _onFileUpdated(self):
        self._filters_tree_controller.showFilters(self._file, self._show_completed)
        self._setIsModified(True)
        self.auto_save()

    def _canExit(self):
        if not self._is_modified:
            return True
        button = self._dialogs.showSaveDiscardCancel(self.tr('Unsaved changes...'))
        if button == QtWidgets.QMessageBox.Save:
            self.save()
            return True
        else:
            return button == QtWidgets.QMessageBox.Discard

    def view_onCloseEvent(self, closeEvent):

        if (self.hasTrayIcon and int(self._settings.value("close_to_tray", 0))):
            self.view.hide()
            closeEvent.ignore()
            return

        if self._canExit():
            if self.filterViewAction.isChecked():  # we only save size if it is visible
                self._settings.setValue("splitter_pos", self.view.centralWidget().sizes())
            self._settings.setValue("current_filters", self._filters_tree_controller.view.getSelectedFilterNames())
            self._settings.setValue("main_window_geometry", self.view.saveGeometry())
            self._settings.setValue("main_window_state", self.view.saveState())

            closeEvent.accept()
        else:
            closeEvent.ignore()

    def _setIsModified(self, is_modified):
        self._is_modified = is_modified
        self._updateTitle()
        self._menu_controller.saveAction.setEnabled(is_modified)
        self._menu_controller.revertAction.setEnabled(is_modified)

    def save(self):
        logger.debug('MainController.save called.')
        self._fileObserver.clear()
        filename = self._file.filename
        ok = True
        if not filename:
            (filename, ok) = \
                QtWidgets.QFileDialog.getSaveFileName(self.view, filter=FILENAME_FILTERS)
        if ok and filename:
            self._file.save(filename)
            self._settings.setValue("last_open_file", filename)
            self._settings.sync()
            self._setIsModified(False)
            logger.debug('Adding {} to watchlist'.format(filename))
            self._fileObserver.addPath(self._file.filename)

    def _updateTitle(self):
        title = 'QTodoTxt - '
        if self._file.filename:
            filename = os.path.basename(self._file.filename)
            title += filename
        else:
            title += 'Untitled'
        if self._is_modified:
            title += ' (*)'
        self.view.setWindowTitle(title)

    def open(self):
        (filename, ok) = \
            QtWidgets.QFileDialog.getOpenFileName(self.view, filter=FILENAME_FILTERS)

        if ok and filename:
            try:
                self.openFileByName(filename)
            except ErrorLoadingFile as ex:
                self._dialogs.showError(str(ex))

    def new(self):
        if self._canExit():
            self._file = File()
            self._loadFileToUI()

    def revert(self):
        if self._dialogs.showConfirm(self.tr('Revert to saved file (and lose unsaved changes)?')):
            try:
                self.openFileByName(self._file.filename)
            except ErrorLoadingFile as ex:
                self._dialogs.showError(str(ex))

    def openFileByName(self, filename):
        logger.debug('MainController.openFileByName called with filename="{}"'.format(filename))
        self._fileObserver.clear()
        try:
            self._file.load(filename)
        except Exception as ex:
            currentfile = self._settings.value("last_open_file", "")
            if currentfile == filename:
                self._dialogs.showError(self.tr("Current file '{}' is not available.\nException: {}").
                                        format(filename, ex))
            else:
                self._dialogs.showError(self.tr("Error opening file: {}.\n Exception:{}").format(filename, ex))
            return
        self._loadFileToUI()
        self._settings.setValue("last_open_file", filename)
        self._settings.sync()
        logger.debug('Adding {} to watchlist'.format(filename))
        self._fileObserver.addPath(self._file.filename)
        self.updateRecentFile()

    def updateRecentFile(self):
        lastOpenedArray = self._menu_controller.getRecentFileNames()
        if self._file.filename in lastOpenedArray:
            lastOpenedArray.remove(self._file.filename)
        lastOpenedArray = lastOpenedArray[:self._menu_controller.maxRecentFiles]
        lastOpenedArray.insert(0, self._file.filename)
        self._settings.setValue("lastOpened", lastOpenedArray[: self._menu_controller.maxRecentFiles])
        self._menu_controller.updateRecentFileActions()

    def _loadFileToUI(self):
        self._setIsModified(False)
        self._filters_tree_controller.showFilters(self._file, self._show_completed)

    def _updateView(self):
        wgeo = self._settings.value("main_window_geometry", None)
        if wgeo is not None:
            self.view.restoreGeometry(wgeo)
        wstate = self._settings.value("main_window_state", None)
        if wstate is not None:
            self.view.restoreState(wstate)
        splitterPosition = self._settings.value("splitter_pos", (200, 400))
        splitterPosition = [int(x) for x in splitterPosition]
        self.view.centralWidget().setSizes(splitterPosition)
        self._restoreShowCompleted()
        self._restoreFilterView()
        self._restoreShowFuture()
        self._restoreShowToolBar()
        self._restoreShowSearch()

    def _restoreShowCompleted(self):
        val = int(self._settings.value("show_completed_tasks", 1))
        if val:
            self._show_completed = True
            self.showCompletedAction.setChecked(True)
        else:
            self._show_completed = False
            self.showCompletedAction.setChecked(False)

    def _restoreShowToolBar(self):
        val = int(self._settings.value("show_toolbar", 1))
        if val:
            self._toolbar_visibility_changed(1)
            self.showToolBarAction.setChecked(True)
        else:
            self._toolbar_visibility_changed(0)
            self.showToolBarAction.setChecked(False)

    def _restoreShowSearch(self):
        val = int(self._settings.value("show_search", 1))
        if val:
            self.view.tasks_view.tasks_search_view.setVisible(True)
            self.showSearchAction.setChecked(True)
        else:
            self.view.tasks_view.tasks_search_view.setVisible(False)
            self.showSearchAction.setChecked(False)

    def updateFilters(self):
        self._onFilterSelectionChanged(self._filters_tree_controller.view.getSelectedFilters())

    def toggleVisible(self):
        if self.view.isMinimized() or self.view.isHidden():
            self.view.show()
            self.view.activateWindow()
        else:
            self.view.hide()

    def anotherInstanceEvent(self, dir):
        tFile = dir + "/qtodo.tmp"
        if not os.path.isfile(tFile):
            return
        time.sleep(0.01)
        f = open(tFile, 'r+b')
        line = f.readline()
        line = line.strip()
        if line == b"1":
            self.view.show()
            self.view.activateWindow()
        if line == b"2":
            self.view.show()
            self.view.activateWindow()
            self._tasks_list_controller.createTask()

        f.close()
        os.remove(tFile)



from PyQt5.QtCore import pyqtSignal, Qt, QSize
from PyQt5.QtWidgets import QWidget, QStackedLayout, QListWidget, QLabel, QListView, QListWidgetItem,\
     QAbstractItemView

from qtodotxt.lib import tasklib
from qtodotxt.ui.dialogs.taskeditor_lineedit import TaskEditorLineEdit


class TaskWidget(QWidget):

    taskModified = pyqtSignal(tasklib.Task)
    taskDeleted = pyqtSignal(tasklib.Task)
    taskCreated = pyqtSignal(tasklib.Task)

    def __init__(self, parent, task, new=False):
        QWidget.__init__(self, parent)
        self.task = task
        self.new = new
        self.layout = QStackedLayout(self)

        self.label = QLabel(self)
        self.label.setTextFormat(Qt.RichText)
        self.label.setOpenExternalLinks(True)
        self._update()
        self.label.setWordWrap(True)
        self.layout.addWidget(self.label)

        self.editor = TaskEditorLineEdit(self, parent.mfile)
        self.layout.addWidget(self.editor)
        self.setLayout(self.layout)

        self.layout.setCurrentIndex(0)
        self.editor.focusOut.connect(self.editFinished)
        self.task.modified.connect(self._update)

    def sizeHint(self):
        new_height = self.label.heightForWidth(self.parent().size().width())
        new_height += 10
        return QSize(self.parent().size().width(), new_height)

    def edit(self):
        self.editor.setText(self.task.text)
        self.layout.setCurrentIndex(1)
        self.editor.setFocus()

    def editFinished(self):
        # qt bug, this method may be called several times for one edit
        self.layout.setCurrentIndex(0)
        text = self.editor.text()
        if not text:
            self.taskDeleted.emit(self.task)
        elif self.new:
            self.task.text = text
            self.new = False
            self.taskCreated.emit(self.task)
        elif text != self.task.text:
            self.task.text = text
            self.taskModified.emit(self)
        self.parent().setFocus()

    def _update(self):
        self.label.setText(self.task.toHtml())

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            self.editor.setText(self.task.text)
            self.layout.setCurrentIndex(0)
            self.parent().setFocus()
            return
        elif event.key() in (Qt.Key_Enter, Qt.Key_Return):
            self.editFinished()
            return
        QWidget.keyPressEvent(self, event)


class TasksListView(QListWidget):

    taskActivated = pyqtSignal(tasklib.Task)
    currentTaskChanged = pyqtSignal(list)
    taskModified = pyqtSignal(tasklib.Task)
    taskCreated = pyqtSignal(tasklib.Task)
    taskDeleted = pyqtSignal(tasklib.Task)

    def __init__(self, parent):
        QListWidget.__init__(self, parent)
        self.mfile = None
        self.setAlternatingRowColors(True)
        self.setLayoutMode(QListView.Batched)
        self.setResizeMode(QListView.Adjust)
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.activated.connect(self._taskActivated)
        self.setUniformItemSizes(False)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

    def resizeEvent(self, ev):
        """
        Why do we need to do that???
        """
        for i in range(self.count()):
            item = self.item(i)
            widg = self.itemWidget(item)
            if widg:
                item.setSizeHint(widg.sizeHint())

    def setFileObject(self, mfile):
        self.mfile = mfile

    def _taskActivated(self, idx):
        self.taskActivated.emit(self.taskFromIndex(idx))

    def taskFromIndex(self, idx):
        it = self.itemFromIndex(idx)
        if it:
            return self.itemWidget(it).task

    def currentChanged(self, current, previous):
        it = self.itemFromIndex(current)
        self.scrollTo(current)  # maybe should be in task controller
        widg = self.itemWidget(it)
        if widg:
            self.currentTaskChanged.emit([widg.task])

    def createTask(self, template_task=None):
        """
        Add a task and start editing it with inline editor
        """
        if isinstance(template_task, tasklib.Task):
            task = tasklib.Task(template_task.text)
        else:
            task = tasklib.Task("")
        idxs = self.selectedIndexes()
        item = self.addTask(task, idxs, new=True)
        self.scrollToItem(item)
        self.itemWidget(item).edit()

    def items(self):
        """
        iterate over all items
        """
        for i in range(self.count()):
            yield self.item(i)

    def tasks(self):
        """
        iterate over all tasks
        """
        for i in range(self.count()):
            yield self.indexWidget(i).task

    def addTask(self, task, idxs=None, new=False):
        """
        add a new task to the view
        """
        item = QListWidgetItem()
        twidget = TaskWidget(self, task, new=new)
        item.setSizeHint(twidget.sizeHint())
        if not idxs:
            self.addItem(item)
        else:
            row = idxs[-1].row()
            row += 1
            self.insertItem(row, item)
        self.setItemWidget(item, twidget)
        twidget.taskModified.connect(self.taskModified.emit)
        twidget.taskDeleted.connect(self._taskDeleted)
        twidget.taskCreated.connect(self.taskCreated.emit)
        return item

    def findItemByTask(self, task):
        for it in self.items():
            if task == self.itemWidget(it).task:
                return it
        return None

    def selectTask(self, task):
        for item in self.items():
            if self.itemWidget(item).task == task:
                self.setCurrentItem(item)
                return

    def selectTaskByText(self, text):
        for item in self.items():
            if self.itemWidget(item).task.text == text:
                self.setCurrentItem(item)
                return

    def removeTask(self, task):
        for item in self.items():
            if self.itemWidget(item).task == task:
                self.takeItem(self.indexFromItem(item).row())
                return

    def _taskDeleted(self, task):
        emit = False
        it = self.findItemByTask(task)
        widg = self.itemWidget(it)
        if not widg:  # can happen due to qt bug mentiooned above
            return
        if not self.itemWidget(it).new:
            emit = True
        self.removeTask(task)
        if emit:
            self.taskDeleted.emit(task)

    def getSelectedTasks(self):
        idxs = self.selectedIndexes()
        return [self.indexWidget(idx).task for idx in idxs]

    def editCurrentTask(self):
        it = self.currentItem()
        if it:
            self.itemWidget(it).edit()

    def currentTask(self):
        it = self.currentItem()
        if it:
            return self.itemWidget(it).task
        return None

    def selectNext(self):
        idx = self.currentIndex()
        row = idx.row() + 1
        print("ROW", row, self.count())
        if row >= self.count():
            print("HERE")
            row = self.count() - 2
        idx = idx.sibling(row, 0)
        self.setCurrentIndex(idx)
        print(self.currentTask())



import sys

from PyQt5 import QtCore, QtGui, QtWidgets

from qtodotxt.ui.dialogs.shortcutsui import Ui_ShortcutsUI


class Shortcuts(QtWidgets.QDialog):

    def __init__(self, maincontroller):
        QtWidgets.QDialog.__init__(self, maincontroller.view)
        self.maincontroller = maincontroller
        self.ui = Ui_ShortcutsUI()
        self.ui.setupUi(self)
        self.settings = QtCore.QSettings()
        self.setWindowFlags(QtCore.Qt.Dialog
                            | QtCore.Qt.MSWindowsFixedSizeDialogHint
                            | QtCore.Qt.WindowStaysOnBottomHint
                            | QtCore.Qt.WindowSystemMenuHint
                            | QtCore.Qt.WindowTitleHint
                            | QtCore.Qt.WindowCloseButtonHint)

        self.model = QtGui.QStandardItemModel(0, 2, self)
        self.model.setHeaderData(0, 1, self.tr("Comment"))
        self.model.setHeaderData(1, 1, self.tr("Shortcut"))

        self.ui.tableView.setModel(self.model)

        self.addShortcuts(self.maincontroller.findChildren(QtWidgets.QAction))
        self.addShortcuts(self.maincontroller.view.findChildren(QtWidgets.QAction))
        self.addShortcuts(self.maincontroller._tasks_list_controller.findChildren(QtWidgets.QAction))
        self.addShortcuts(self.maincontroller._menu_controller.findChildren(QtWidgets.QAction))

        self.ui.tableView.resizeColumnsToContents()
        self.ui.tableView.setAlternatingRowColors(True)
        self.ui.tableView.setColumnWidth(1, 200)

        self.ui.tableView.setColumnWidth(0, 230)
        self.ui.tableView.setColumnWidth(1, 230)

        self.ui.closeButton.clicked.connect(self.close)

    def addShortcuts(self, list):
        row = self.model.rowCount()
        for action in list:
            if action.shortcut().toString() != "":
                comment = action.text()
                for shortcut in action.shortcuts():
                    self.model.insertRow(row)
                    self.model.setData(self.model.index(row, 0), comment)
                    self.model.setData(self.model.index(row, 1), shortcut.toString())
                    row += 1

    def closeEvent(self, event):
        self.deleteLater()
        self.maincontroller.view.show()


if __name__ == "__main__":
    QtCore.QCoreApplication.setOrganizationName("QTodoTxt")
    QtCore.QCoreApplication.setApplicationName("QTodoTxt")
    app = QtGui.QApplication(sys.argv)
    s = Shortcuts(None)
    s.show()
    sys.exit(app.exec_())



from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets

from qtodotxt.ui.dialogs import about_dialog
from qtodotxt.ui.dialogs.settings import Settings
from qtodotxt.ui.dialogs.shortcuts import Shortcuts


class MenuController(QtCore.QObject):

    def __init__(self, main_controller, menu):
        super(MenuController, self).__init__()
        self._main_controller = main_controller
        self._menu = menu
        self.maxRecentFiles = 3  # constant determines the number of files stored in the history
        self.recentFileArray = []
        self._initMenuBar()

    def _initMenuBar(self):
        self._initFileMenu()
        self._initEditMenu()
        self._initViewMenu()
        self._initHelpMenu()

    def _initFileMenu(self):
        fileMenu = self._menu.addMenu(self.tr('&File'))
        fileMenu.addAction(self._createNewAction())
        fileMenu.addAction(self._createOpenAction())

        lastOpened = fileMenu.addMenu(QtGui.QIcon(self._main_controller.view.style +
                                                  '/resources/FileOpen.png'), "Open &Recent")
        for ind in range(self.maxRecentFiles):
            self.recentFileArray.append(QtWidgets.QAction(self, visible=False, triggered=self.openRecentFile))
            lastOpened.addAction(self.recentFileArray[ind])

        fileMenu.addAction(self._createSaveAction())
        fileMenu.addAction(self._createRevertAction())
        fileMenu.addSeparator()
        fileMenu.addAction(self._createPreferenceAction())
        fileMenu.addSeparator()
        fileMenu.addAction(self._createExitAction())

    def updateRecentFileActions(self):
        recentFileNames = self.getRecentFileNames()
        ind = 1
        for i in range(len(recentFileNames)):
            text = "&%d %s" % (ind, recentFileNames[i])
            self.recentFileArray[i].setText(text)
            self.recentFileArray[i].setIcon(QtGui.QIcon(self._main_controller.view.style +
                                                        '/resources/FileOpen.png'))
            self.recentFileArray[i].setData(recentFileNames[i])
            self.recentFileArray[i].setVisible(True)
            self.recentFileArray[i].setShortcuts(["Ctrl+" + str(ind)])
            ind += 1

    def getRecentFileNames(self):
        recentFileNames = []
        recentFileNames.append(QtCore.QSettings().value("lastOpened", []))
        return recentFileNames[0]

    def openRecentFile(self):
        action = self.sender()
        if action:
            self._main_controller.openFileByName(action.data())

    def _initEditMenu(self):
        editMenu = self._menu.addMenu(self.tr('&Edit'))
        tlc = self._main_controller._tasks_list_controller
        editMenu.addAction(tlc.createTaskAction)
        editMenu.addAction(tlc.createTaskActionOnTemplate)
        editMenu.addAction(tlc.editTaskAction)
        editMenu.addAction(tlc.copySelectedTasksAction)
        editMenu.addSeparator()
        editMenu.addAction(tlc.completeSelectedTasksAction)
        if int(QtCore.QSettings().value("show_delete", 1)):
            editMenu.addAction(tlc.deleteSelectedTasksAction)
        editMenu.addSeparator()
        editMenu.addAction(tlc.increasePrioritySelectedTasksAction)
        editMenu.addAction(tlc.decreasePrioritySelectedTasksAction)

    def _initViewMenu(self):
        viewMenu = self._menu.addMenu(self.tr('&View'))
        viewMenu.addAction(self._main_controller.showToolBarAction)
        viewMenu.addSeparator()
        viewMenu.addAction(self._main_controller.filterViewAction)
        viewMenu.addAction(self._main_controller.showFutureAction)
        viewMenu.addAction(self._main_controller.showCompletedAction)
        viewMenu.addAction(self._main_controller.showSearchAction)

    def _initHelpMenu(self):
        helpMenu = self._menu.addMenu(self.tr('&Help'))
        helpMenu.addAction(self._createAboutAction())
        helpMenu.addAction(self._createShortcutsListAction())

    def _createAboutAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/ApplicationAbout.png'), '&About', self)
        action.setShortcuts(["F1"])
        action.triggered.connect(self._about)
        return action

    def _createShortcutsListAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/ApplicationAbout.png'),
                                   self.tr('&Shortcuts list'), self)
        action.setShortcuts(["Ctrl+F1"])
        action.triggered.connect(self._show_shortcuts)
        return action

    def _about(self):
        about_dialog.show(self._menu)

    def _show_shortcuts(self):
        shortcuts = Shortcuts(self._main_controller)
        shortcuts.setModal(True)
        shortcuts.show()

    def _createNewAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/FileNew.png'), self.tr('&New'), self)
        # infrequent action, I prefer to use ctrl+n for new task.
        action.setShortcuts(["Ctrl+Shift+N"])
        action.triggered.connect(self._main_controller.new)
        self.newAction = action
        return action

    def _createOpenAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/FileOpen.png'), self.tr('&Open'), self)
        action.setShortcuts(["Ctrl+O"])
        action.triggered.connect(self._main_controller.open)
        self.openAction = action
        return action

    def _createSaveAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/FileSave.png'), self.tr('&Save'), self)
        action.setShortcuts(["Ctrl+S"])
        action.triggered.connect(self._main_controller.save)
        self.saveAction = action
        return action

    def _createRevertAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/FileRevert.png'), self.tr('&Revert'), self)
        action.triggered.connect(self._main_controller.revert)
        self.revertAction = action
        return action

    def _createPreferenceAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/ApplicationPreferences.png'),
                                   self.tr('&Preferences'), self)
        action.triggered.connect(self._show_preferences)
        self.preferencesAction = action
        return action

    def _createExitAction(self):
        action = QtWidgets.QAction(QtGui.QIcon(self._main_controller.view.style +
                                               '/resources/ApplicationExit.png'), self.tr('E&xit'), self)
        action.setShortcuts(["Ctrl+Q"])
        action.triggered.connect(self._main_controller.exit)
        return action

    def _show_preferences(self):
        settings = Settings(self._main_controller)
        # simplest way to stop the preferences dialog from being left on screen
        #  when the main app window is hidden is to create it as a modal dialog
        settings.setModal(True)
        settings.show()



from PyQt5 import QtCore
from qtodotxt.lib.filters import ContextFilter, ProjectFilter, DueTodayFilter, DueTomorrowFilter, DueThisWeekFilter, \
    DueThisMonthFilter, DueOverdueFilter, PriorityFilter

# class IFiltersTreeView(object):
#    def addFilter(self, filter): pass
#    def clear(self): pass
#    def clearSelection(self): pass
#    def selectFilter(self, filter): pass
#    def getSelectedFilters(self): pass
#    filterSelectionChanged = QtCore.pyqtSignal()
#    def selectAllTasksFilter(self): pass


class FiltersTreeController(QtCore.QObject):

    filterSelectionChanged = QtCore.pyqtSignal(list)

    def __init__(self, view):
        QtCore.QObject.__init__(self)
        self.view = view
        self.view.filterSelectionChanged.connect(self.view_filterSelectionChanged)
        self._is_showing_filters = False

    def view_filterSelectionChanged(self, filters):
        if not self._is_showing_filters:
            self.filterSelectionChanged.emit(filters)

    def showFilters(self, file, show_completed=False):
        self._is_showing_filters = True
        previouslySelectedFilters = self.view.getSelectedFilters()
        self.view.clearSelection()
        self.view.clear()
        self._addAllContexts(file, show_completed)
        self._addAllProjects(file, show_completed)
        self._addAllDueRanges(file, show_completed)
        self._addAllPriorities(file, show_completed)
        self._updateCounter(file, show_completed)
        self._is_showing_filters = False
        self._reselect(previouslySelectedFilters)

    def _updateCounter(self, file, show_completed=False):
        rootCounters = file.getTasksCounters()
        self.view.updateTopLevelTitles(rootCounters, show_completed)

    def _addAllContexts(self, file, show_completed):
        contexts = file.getAllContexts(show_completed)
        for context, number in contexts.items():
            filter = ContextFilter(context)
            self.view.addFilter(filter, number)

    def _addAllProjects(self, file, show_completed):
        projects = file.getAllProjects(show_completed)
        for project, number in projects.items():
            filter = ProjectFilter(project)
            self.view.addFilter(filter, number)

    def _addAllPriorities(self, file, show_completed):
        priorities = file.getAllPriorities(show_completed)
        for priority, number in priorities.items():
            filter = PriorityFilter(priority)
            self.view.addFilter(filter, number)

    def _addAllDueRanges(self, file, show_completed):

        dueRanges, rangeSorting = file.getAllDueRanges(show_completed)

        for range, number in dueRanges.items():
            if range == 'Today':
                filter = DueTodayFilter(range)
                sortKey = rangeSorting['Today']
            elif range == 'Tomorrow':
                filter = DueTomorrowFilter(range)
                sortKey = rangeSorting['Tomorrow']
            elif range == 'This week':
                filter = DueThisWeekFilter(range)
                sortKey = rangeSorting['This week']
            elif range == 'This month':
                filter = DueThisMonthFilter(range)
                sortKey = rangeSorting['This month']
            elif range == 'Overdue':
                filter = DueOverdueFilter(range)
                sortKey = rangeSorting['Overdue']

            self.view.addDueRangeFilter(filter, number, sortKey)

    def _reselect(self, previouslySelectedFilters):
        for filter in previouslySelectedFilters:
            self.view.selectFilter(filter)
        if not self.view.getSelectedFilters():
            self.view.selectAllTasksFilter()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import logging
import sys
import os

from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets
# import resource for darkstyle
# taken from https://github.com/ColinDuquesnoy/QDarkStyleSheet
import qtodotxt.ui.pyqt5_style_rc  # noqa: F401
import qtodotxt.ui.qTodoTxt_style_rc  # noqa: F401
import qtodotxt.ui.qTodoTxt_dark_style_rc  # noqa: F401

from qtodotxt.ui.controllers.main_controller import MainController
from qtodotxt.ui.dialogs.misc_dialogs import Dialogs
from qtodotxt.ui.views.main_view import MainView
from qtodotxt.lib.file import FileObserver
from qtodotxt.lib.tendo_singleton import SingleInstance


class TrayIcon(QtWidgets.QSystemTrayIcon):
    def __init__(self, main_controller):
        self._controller = main_controller
        self.style = ":/white_icons"
        if str(QtCore.QSettings().value("color_schem", "")).find("dark") >= 0:
            self.style = ":/dark_icons"
        self._initIcon()

    def _initIcon(self):
        view = self._controller.getView()

        icon = QtGui.QIcon(self.style + "/resources/qtodotxt.png")
        QtWidgets.QSystemTrayIcon.__init__(self, icon, view)
        self.activated.connect(self._onActivated)
        self.setToolTip('QTodoTxt')

        menu = QtWidgets.QMenu(self._controller.view)
        create_task_action = menu.addAction(QtGui.QIcon(self.style + "/resources/TaskCreate.png"),
                                            self.tr("Create New Task"))
        create_task_action.triggered.connect(self._createTask)
        toggle_visible_action = menu.addAction(self.tr("Show/Hide Window"))
        toggle_visible_action.triggered.connect(self._controller.toggleVisible)
        exit_action = menu.addAction(QtGui.QIcon(self.style + '/resources/ApplicationExit.png'), self.tr("Exit"))
        exit_action.triggered.connect(self._controller.exit)
        self.setContextMenu(menu)

    def _onActivated(self, activation_reason):
        # Tray Icon has been activated.
        # [0] QSystemTrayIcon.Unknown       Unknown reason
        # [1] QSystemTrayIcon.Context       The context menu for the system tray entry was requested
        # [2] QSystemTrayIcon.DoubleClick   The system tray entry was double clicked
        # [3] QSystemTrayIcon.Trigger       The system tray entry was clicked
        # [4] QSystemTrayIcon.MiddleClick   The system tray entry was clicked with the middle mouse button
        if activation_reason == QtWidgets.QSystemTrayIcon.Trigger:
            if (int(QtCore.QSettings().value("enable_tray", 0)) and
                    int(QtCore.QSettings().value("hide_to_tray", 0))):
                self._controller.toggleVisible()
            else:
                self._createTask()

    def _createTask(self):

        self._controller.view.show()
        self._controller._tasks_list_controller.createTask()


def _parseArgs():
    if len(sys.argv) > 1 and sys.argv[1].startswith('-psn'):
        del sys.argv[1]
    parser = argparse.ArgumentParser(description='QTodoTxt')
    parser.add_argument('file', type=str, nargs='?', metavar='TEXTFILE',
                        help='open the specified file')
    parser.add_argument('-q', '--quickadd', action='store_true',
                        help='opens the add task dialog and exit the application when done')
    parser.add_argument('-l', '--loglevel', type=str, nargs=1, metavar='LOGLEVEL', default=['WARN'],
                        choices=['DEBUG', 'INFO', 'WARNING', 'WARN', 'ERROR', 'CRITICAL'],
                        help='set one of these logging levels: DEBUG, INFO, WARNING, ERROR, CRITICAL')
    return parser.parse_args()


def _setupLogging(loglevel):
    numeric_level = getattr(logging, loglevel[0].upper(), None)
    if isinstance(numeric_level, int):
        logging.basicConfig(format='{asctime}.{msecs:.0f} [{name}] {levelname}: {message}',
                            level=numeric_level, style='{', datefmt='%H:%M:%S')


def _createController(args):
    window = MainView()
    dialogs = Dialogs(window, 'QTodoTxt')
    return MainController(window, dialogs, args)


def setupAnotherInstanceEvent(controller, dir):
    fileObserver = FileObserver(controller, dir)
    fileObserver.addPath(dir)
    fileObserver.dirChangetSig.connect(controller.anotherInstanceEvent)


def setupSingleton(args, me):
    dir = os.path.dirname(sys.argv[0])
    tempFileName = dir + "/qtodo.tmp"
    if me.initialized is True:
        if os.path.isfile(tempFileName):
            os.remove(tempFileName)
    else:
        f = open(tempFileName, 'w')
        if args.quickadd is False:
            f.write("1")
        if args.quickadd is True:
            f.write("2")
        f.flush()
        f.close()
        sys.exit(-1)


def run():
    # First set some application settings for QSettings
    QtCore.QCoreApplication.setOrganizationName("QTodoTxt")
    QtCore.QCoreApplication.setApplicationName("QTodoTxt")
    # Now set up our application and start
    app = QtWidgets.QApplication(sys.argv)

    name = QtCore.QLocale.system().name()
    translator = QtCore.QTranslator()
    if translator.load(str(name) + ".qm", "..//i18n"):
        app.installTranslator(translator)

    args = _parseArgs()

    dir = os.path.dirname(sys.argv[0])
    needSingleton = QtCore.QSettings().value("singleton", 0)

    # clear or write to TMP file, of main instance
    if int(needSingleton):
        me = SingleInstance()
        setupSingleton(args, me)

    _setupLogging(args.loglevel)
    #    logger = logging.getLogger(__file__[:-3]) # in case someone wants to log here
    controller = _createController(args)

    # Connecting to a processor reading TMP file
    if needSingleton:
        setupAnotherInstanceEvent(controller, dir)

    controller.show()
    if int(QtCore.QSettings().value("enable_tray", 0)):
        # If the controller.show() method is not called, the todo.txt file
        #  is not loaded.  If the controller.show() method is modified to do the
        #  initial setup but not show the main view (or to show the main view
        #  *after* the setup is done) then the layout is borked.
        # This is a simple solution that solves the immediate problem, but a
        #  rewrite of the initialization code that affords a better solution
        #  would be worth considering at some point.
        if int(QtCore.QSettings().value("hide_on_startup", 0)):
            controller.view.hide()
        icon = TrayIcon(controller)
        icon.show()
        controller.hasTrayIcon = True
    app.exec_()
    sys.exit()


if __name__ == "__main__":
    run()



import unittest
from tempfile import mkstemp
from os import remove
from datetime import date, datetime, timedelta
from sys import version

from qtodotxt.lib.file import File, ErrorLoadingFile
from qtodotxt.lib.tasklib import Task


PYTHON_VERSION = version[:3]


if PYTHON_VERSION < '3.3':
    FileNotFoundError = BaseException


class TestFile(unittest.TestCase):
    def setUp(self):
        self.file = File()
        self.tmpfile = mkstemp(text=True)[1]

    def tearDown(self):
        try:
            remove(self.tmpfile)
        except FileNotFoundError:
            pass
        except OSError as ex:    # maintain compatibility with Python 3.2
            if ex.errno != 2:
                raise
        except:
            raise

    def saveAndReload(self):
        self.file.save(self.tmpfile)
        self.file = File()
        self.file.load(self.tmpfile)

    def test_single_task(self):
        text = 'due:1999-10-10 do something +project1 @context1'
        self.file.tasks.append(Task(text))
        self.saveAndReload()
        self.assertEqual(self.file.tasks[0].text, text)
        self.assertEqual(self.file.tasks[0].contexts, ['context1'])
        self.assertEqual(self.file.tasks[0].projects, ['project1'])
        self.assertFalse(self.file.tasks[0].is_complete)
        self.assertFalse(self.file.tasks[0].priority)
        self.assertEqual(self.file.tasks[0].due, datetime(1999, 10, 10))

    def test_two_tasks(self):
        task1 = 'do something +project1 @context1'
        task2 = '(A) do something else +project1 @context2'
        self.file.tasks.extend([
            Task(task1),
            Task(task2)
        ])
        self.saveAndReload()
        self.assertEqual(self.file.tasks[0].text, task2)
        self.assertEqual(self.file.tasks[1].text, task1)

    def test_five_tasks(self):
        task1 = Task('do something +project1 @context1')
        task2 = Task('(A) do something else +project1 @context2')
        task3 = Task('do something else +project1 @context2')
        task4 = Task('something else +project1 @context2')
        task5 = Task('abc +project1 @context2')
        self.file.tasks.extend([task1, task2, task3, task4, task5])
        self.saveAndReload()
        self.assertEqual(self.file.tasks[0].text, task2.text)
        self.assertEqual(self.file.tasks[1].text, task5.text)
        self.assertEqual(self.file.tasks[2].text, task1.text)
        self.assertEqual(self.file.tasks[3].text, task3.text)
        self.assertEqual(self.file.tasks[4].text, task4.text)

    def test_get_all_contexts(self):
        self.file.tasks.extend([
            Task('x task with @context1'),
            Task('task with @context2'),
            Task('task with @context1 and @context2'),
            Task('task with @context1 and @context2 and @context3')
        ])
        self.saveAndReload()
        self.assertEqual(self.file.getAllContexts(), {'context1': 2, 'context2': 3, 'context3': 1})

    def test_get_all_incl_completed_contexts(self):
        self.file.tasks.extend([
            Task('x task with @context1'),
            Task('task with @context2'),
            Task('task with @context1 and @context2'),
            Task('x task with @context1 and @context2 and @context3')
        ])
        self.saveAndReload()
        self.assertEqual(self.file.getAllContexts(True), {'context1': 1, 'context2': 2, 'context3': 0})

    def test_get_all_projects(self):
        self.file.tasks.extend([
            Task('x task with +project1'),
            Task('task with +project2'),
            Task('task with +project1 and +project2'),
            Task('task with +project1 and +project2 and +project3')
        ])
        self.saveAndReload()
        self.assertEqual(self.file.getAllProjects(), {'project1': 2, 'project2': 3, 'project3': 1})

    def test_get_all_due_ranges(self):
        today = date.today().strftime('%Y-%m-%d')
        yesterday = (date.today() - timedelta(days=1)).strftime('%Y-%m-%d')

        self.file.tasks.extend([
            Task('x due:' + today + ' completed task of today'),
            Task('due:' + today + ' first task of today'),
            Task('due:' + today + ' second task of today'),
            Task('due:' + yesterday + ' task of yesterday'),
        ])
        self.saveAndReload()
        self.assertEqual(self.file.getAllDueRanges()[0], {'Today': 2, 'This week': 2, 'This month': 2, 'Overdue': 1})
        self.assertIsInstance(self.file.getAllDueRanges()[1], dict)

    def test_get_all_projects_incl_completed(self):
        self.file.tasks.extend([
            Task('x task with +project1'),
            Task('task with +project2'),
            Task('task with +project1 and +project2'),
            Task('x task with +project1 and +project2 and +project3')
        ])
        self.saveAndReload()
        self.assertEqual(self.file.getAllProjects(True), {'project1': 1, 'project2': 2, 'project3': 0})

    def test_load_empty_filename(self):
        self.assertRaises(ErrorLoadingFile, self.file.load, '')

    def test_load_nonexisting_file(self):
        remove(self.tmpfile)
        self.assertRaises(ErrorLoadingFile, self.file.load, self.tmpfile)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'settingsui.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsUI(object):
    def setupUi(self, SettingsUI):
        SettingsUI.setObjectName("SettingsUI")
        SettingsUI.resize(763, 527)
        self.gridLayout = QtWidgets.QGridLayout(SettingsUI)
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox = QtWidgets.QGroupBox(SettingsUI)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.autoSaveCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.autoSaveCheckBox.setChecked(True)
        self.autoSaveCheckBox.setObjectName("autoSaveCheckBox")
        self.gridLayout_2.addWidget(self.autoSaveCheckBox, 0, 1, 1, 1)
        self.autoArchiveCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.autoArchiveCheckBox.setChecked(True)
        self.autoArchiveCheckBox.setObjectName("autoArchiveCheckBox")
        self.gridLayout_2.addWidget(self.autoArchiveCheckBox, 1, 1, 1, 1)
        self.addCreatedDateCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.addCreatedDateCheckBox.setChecked(True)
        self.addCreatedDateCheckBox.setObjectName("addCreatedDateCheckBox")
        self.gridLayout_2.addWidget(self.addCreatedDateCheckBox, 2, 1, 1, 1)
        self.confirmCompletionCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.confirmCompletionCheckBox.setChecked(True)
        self.confirmCompletionCheckBox.setObjectName("confirmCompletionCheckBox")
        self.gridLayout_2.addWidget(self.confirmCompletionCheckBox, 3, 1, 1, 2)
        self.deleteActionCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.deleteActionCheckBox.setObjectName("deleteActionCheckBox")
        self.gridLayout_2.addWidget(self.deleteActionCheckBox, 4, 1, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 5, 1, 1, 1)
        self.lowestPriorityLineEdit = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lowestPriorityLineEdit.sizePolicy().hasHeightForWidth())
        self.lowestPriorityLineEdit.setSizePolicy(sizePolicy)
        self.lowestPriorityLineEdit.setMaximumSize(QtCore.QSize(16777212, 16777215))
        self.lowestPriorityLineEdit.setObjectName("lowestPriorityLineEdit")
        self.gridLayout_2.addWidget(self.lowestPriorityLineEdit, 5, 2, 1, 1)
        self.trayCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.trayCheckBox.setObjectName("trayCheckBox")
        self.gridLayout_2.addWidget(self.trayCheckBox, 9, 1, 1, 2)
        self.frame = QtWidgets.QFrame(self.groupBox)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout.setContentsMargins(11, 0, 11, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.hideToTrayCheckBox = QtWidgets.QCheckBox(self.frame)
        self.hideToTrayCheckBox.setObjectName("hideToTrayCheckBox")
        self.verticalLayout.addWidget(self.hideToTrayCheckBox)
        self.hideOnStartupCheckBox = QtWidgets.QCheckBox(self.frame)
        self.hideOnStartupCheckBox.setObjectName("hideOnStartupCheckBox")
        self.verticalLayout.addWidget(self.hideOnStartupCheckBox)
        self.closeToTrayCheckBox = QtWidgets.QCheckBox(self.frame)
        self.closeToTrayCheckBox.setObjectName("closeToTrayCheckBox")
        self.verticalLayout.addWidget(self.closeToTrayCheckBox)
        self.gridLayout_2.addWidget(self.frame, 10, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 80, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 10, 4, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(261, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 11, 3, 1, 1)
        self.closeButton = QtWidgets.QPushButton(self.groupBox)
        self.closeButton.setObjectName("closeButton")
        self.gridLayout_2.addWidget(self.closeButton, 12, 4, 1, 1)
        self.singletonCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.singletonCheckBox.setObjectName("singletonCheckBox")
        self.gridLayout_2.addWidget(self.singletonCheckBox, 6, 1, 1, 1)
        self.colorSchemComboBox = QtWidgets.QComboBox(self.groupBox)
        self.colorSchemComboBox.setObjectName("colorSchemComboBox")
        self.colorSchemComboBox.addItem("")
        self.colorSchemComboBox.addItem("")
        self.colorSchemComboBox.addItem("")
        self.gridLayout_2.addWidget(self.colorSchemComboBox, 8, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 8, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)

        self.retranslateUi(SettingsUI)
        QtCore.QMetaObject.connectSlotsByName(SettingsUI)

    def retranslateUi(self, SettingsUI):
        _translate = QtCore.QCoreApplication.translate
        SettingsUI.setWindowTitle(_translate("SettingsUI", "SettingsUI"))
        self.groupBox.setTitle(_translate("SettingsUI", "QTodoTxt Settings"))
        self.autoSaveCheckBox.setText(_translate("SettingsUI", "Autosave"))
        self.autoArchiveCheckBox.setText(_translate("SettingsUI", "AutoArchive"))
        self.addCreatedDateCheckBox.setText(_translate("SettingsUI", "Add created date"))
        self.confirmCompletionCheckBox.setText(_translate("SettingsUI", "Ask for confirmation before task completion"))
        self.deleteActionCheckBox.setText(_translate("SettingsUI", "Show Delete action (Require restart)"))
        self.label.setText(_translate("SettingsUI", "Lowest task priority"))
        self.lowestPriorityLineEdit.setText(_translate("SettingsUI", "D"))
        self.trayCheckBox.setText(_translate("SettingsUI", "Enable system tray (Require restart)"))
        self.hideToTrayCheckBox.setText(_translate("SettingsUI", "Hide to tray"))
        self.hideOnStartupCheckBox.setText(_translate("SettingsUI", "Hide on startupp"))
        self.closeToTrayCheckBox.setText(_translate("SettingsUI", "CheckBox"))
        self.closeButton.setText(_translate("SettingsUI", "Close"))
        self.singletonCheckBox.setText(_translate("SettingsUI", "Singleton mode"))
        self.colorSchemComboBox.setItemText(0, _translate("SettingsUI", "System"))
        self.colorSchemComboBox.setItemText(1, _translate("SettingsUI", "dark_blue"))
        self.colorSchemComboBox.setItemText(2, _translate("SettingsUI", "dark_orange"))
        self.label_2.setText(_translate("SettingsUI", "Theme"))




from datetime import date, timedelta
import collections
import calendar
from collections import OrderedDict
import string

from dateutil.relativedelta import relativedelta

from PyQt5 import QtCore, QtWidgets


def end_of_month(today=None, month=None):
    if today is None:
        today = date.today()
    if month is None:
        month = today.month
    year = today.year
    if month < today.month:
        year += 1
    eom = date(year=year, month=month, day=calendar.monthrange(year, month)[1])
    return eom.isoformat()


def end_of_next_month(today=None):
    if today is None:
        today = date.today()
    today += relativedelta(months=1)
    eom = date(year=today.year, month=today.month, day=calendar.monthrange(today.year, today.month)[1])
    return eom.isoformat()


def end_of_week(day=None):
    if day is None:
        day = date.today()
    return (day + timedelta((6 - day.weekday()) % 7)).isoformat()


def end_of_next_week(day=None):
    if day is None:
        day = date.today()
    return (day + timedelta((13 - day.weekday()) % 14)).isoformat()


def end_of_work_week(day=None):
    if day is None:
        day = date.today()
    if day.weekday() > 4:
        return end_of_next_work_week()
    return (day + timedelta((4 - day.weekday()) % 7)).isoformat()


def end_of_next_work_week(day=None):
    if day is None:
        day = date.today()
    return (day + timedelta((11 - day.weekday()) % 14)).isoformat()


def end_of_year(day=None):
    if day is None:
        day = date.today()
    return (date(year=day.year + 1, month=1, day=1) - timedelta(days=1)).isoformat()


class CompleterSetup(object):
    autocomplete_pairs = collections.OrderedDict([
        ('due:Today', ''),
        ('due:Tomorrow', ''),
        ('due:EndOfWorkWeek', ''),
        ('due:EndOfNextWorkWeek', ''),
        ('due:EndOfWeek', ''),
        ('due:EndOfNextWeek', ''),
        ('due:EndOfMonth', ''),
        ('due:EndOfNextMonth', ''),
        ('due:EndOfYear', ''),
        ('due:January', ''),
        ('due:February', ''),
        ('due:March', ''),
        ('due:April', ''),
        ('due:May', ''),
        ('due:June', ''),
        ('due:July', ''),
        ('due:August', ''),
        ('due:September', ''),
        ('due:October', ''),
        ('due:November', ''),
        ('t:Today', ''),
        ('t:Tomorrow', ''),
        ('t:EndOfWeek', ''),
        ('t:EndOfNextWeek', ''),
        ('t:EndOfMonth', ''),
        ('t:EndOfNextMonth', ''),
        ('t:EndOfYear', ''),
        ('due:December', '')
    ])  # yapf: disable

    def __init__(self, editor):

        self._populateKeys(self.autocomplete_pairs)

    def _populateKeys(self, keys):
        self._populateDues(keys)
        self._populateThresholds(keys)

    def _populateThresholds(self, keys):
        keys['t:Today'] = 't:' + date.today().isoformat()
        keys['t:Tomorrow'] = 't:' + (date.today() + timedelta(days=1)).isoformat()
        keys['t:EndOfWorkWeek'] = 't:' + end_of_work_week()
        keys['t:EndOfNextWorkWeek'] = 't:' + end_of_next_work_week()
        keys['t:EndOfWeek'] = 't:' + end_of_week()
        keys['t:EndOfNextWeek'] = 't:' + end_of_next_week()
        keys['t:EndOfMonth'] = 't:' + end_of_month()
        keys['t:EndOfNextMonth'] = 't:' + end_of_next_month()
        keys['t:EndOfYear'] = 't:' + end_of_year()

    def _populateDues(self, keys):
        keys['due:Today'] = 'due:' + date.today().isoformat()
        keys['due:Tomorrow'] = 'due:' + (date.today() + timedelta(days=1)).isoformat()
        keys['due:EndOfWorkWeek'] = 'due:' + end_of_work_week()
        keys['due:EndOfNextWorkWeek'] = 'due:' + end_of_next_work_week()
        keys['due:EndOfWeek'] = 'due:' + end_of_week()
        keys['due:EndOfNextWeek'] = 'due:' + end_of_next_week()
        keys['due:EndOfMonth'] = 'due:' + end_of_month()
        keys['due:EndOfNextMonth'] = 'due:' + end_of_next_month()
        keys['due:EndOfYear'] = 'due:' + end_of_year()
        keys['due:January'] = "due:" + end_of_month(month=1)
        keys['due:February'] = "due:" + end_of_month(month=2)
        keys['due:March'] = "due:" + end_of_month(month=3)
        keys['due:April'] = "due:" + end_of_month(month=4)
        keys['due:May'] = "due:" + end_of_month(month=5)
        keys['due:June'] = "due:" + end_of_month(month=6)
        keys['due:July'] = "due:" + end_of_month(month=7)
        keys['due:August'] = "due:" + end_of_month(month=8)
        keys['due:September'] = "due:" + end_of_month(month=9)
        keys['due:October'] = "due:" + end_of_month(month=10)
        keys['due:November'] = "due:" + end_of_month(month=11)
        keys['due:December'] = "due:" + end_of_month(month=12)


class ValuesGenerator(object):
    def __init__(self, mfile, keys):
        self._mfile = mfile
        self._dates = keys
        self._priorities = ["(" + i + ")" for i in string.ascii_uppercase]
        self._values = []
        self._completedValues = []
        self._values.extend(self._priorities)
        self.updateValues()

    def updateTodoValues(self):
        contexts = self._mfile.getAllContexts()
        projects = self._mfile.getAllProjects()
        for context in contexts:
            self._values.append('@' + context)
        for project in projects:
            self._values.append('+' + project)

    def updateCompletedValues(self):
        contexts = self._mfile.getAllContexts(True)
        projects = self._mfile.getAllProjects(True)
        for context in contexts:
            self._completedValues.append('@' + context)
        for project in projects:
            self._completedValues.append('+' + project)

    def updateValues(self):
        self.updateTodoValues()
        self.updateCompletedValues()

    def get_values(self):
        return sorted(list(OrderedDict.fromkeys(self._completedValues + self._values))) + self._dates


class TaskEditorLineEdit(QtWidgets.QTextEdit):

    focusOut = QtCore.pyqtSignal()

    def __init__(self, parent, mfile):
        QtWidgets.QTextEdit.__init__(self, parent)
        self.mfile = mfile
        self._values_generator = ValuesGenerator(mfile, list(CompleterSetup.autocomplete_pairs.keys()))
        model = self._values_generator.get_values()
        self._completerSetup = CompleterSetup(self)
        self._completer = QtWidgets.QCompleter(model)
        self._completer.setWidget(self)
        self._completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
        self._completer.activated.connect(self._insertCompletion)
        self._keysToIgnore = [QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return, QtCore.Qt.Key_Escape, QtCore.Qt.Key_Tab]

    def text(self):
        return self.toPlainText()

    def updateValues(self):
        self._values_generator.updateValues()

    def _insertCompletion(self, completion):
        """
        This is the event handler for the QCompleter.activated(QString) signal,
        it is called when the user selects an item in the completer popup.
        """
        currentText = self.text()
        cursor = self.textCursor()
        completionPrefixSize = len(self._completer.completionPrefix())
        textFirstPart = cursor.position() - completionPrefixSize
        textLastPart = textFirstPart + completionPrefixSize

        if completion in self._completerSetup.autocomplete_pairs:
            completion = self.replaceAutocompleteKeys(completion)

        newtext = currentText[:textFirstPart] + completion + " " + currentText[textLastPart:]
        newCursorPos = cursor.position() + (len(completion) - completionPrefixSize) + 1

        self.setText(newtext)
        cursor.setPosition(newCursorPos)
        self.setTextCursor(cursor)

    def replaceAutocompleteKeys(self, completion):
        if completion in self._completerSetup.autocomplete_pairs.keys():
            return self._completerSetup.autocomplete_pairs[completion]

    def textUnderCursor(self):
        text = self.text()
        textUnderCursor = ''
        i = self.textCursor().position() - 1
        while i >= 0 and text[i] != " ":
            textUnderCursor = text[i] + textUnderCursor
            i -= 1
        return textUnderCursor

    def focusOutEvent(self, ev):
        super(TaskEditorLineEdit, self).focusOutEvent(ev)
        self.focusOut.emit()

    def keyPressEvent(self, event):
        if event.key() in self._keysToIgnore:
            event.ignore()
            return
        super(TaskEditorLineEdit, self).keyPressEvent(event)
        completionPrefix = self.textUnderCursor()
        if completionPrefix != self._completer.completionPrefix():
            self._updateCompleterPopupItems(completionPrefix)
        if len(event.text()) > 0 and len(completionPrefix) > 0:
            if event.key() not in self._keysToIgnore:
                cr = self.cursorRect()
                cr.setWidth(self._completer.popup().sizeHintForColumn(0) + self._completer.popup().verticalScrollBar()
                            .sizeHint().width())
                self._completer.complete(cr)
        if len(completionPrefix) == 0:
            self._completer.popup().hide()

    def _updateCompleterPopupItems(self, completionPrefix):
        """
        Filters the completer's popup items to only show items
        with the given prefix.
        """
        self._completer.setCompletionPrefix(completionPrefix)
        self._completer.popup().setCurrentIndex(self._completer.completionModel().index(0, 0))



import sys
from PyQt5 import QtWidgets
from qtodotxt.ui.dialogs.taskeditor_lineedit import TaskEditorLineEdit


class TaskEditorDialog(QtWidgets.QDialog):

    def __init__(self, parent, mfile):
        QtWidgets.QDialog.__init__(self)
        self._mfile = mfile
        self._initUI()

    def _initUI(self):
        self.setWindowTitle("Task Editor")
        vbox = QtWidgets.QVBoxLayout()

        self._label = QtWidgets.QLabel("Task:")
        vbox.addWidget(self._label)

        self._edit = TaskEditorLineEdit(self, self._mfile)
        vbox.addWidget(self._edit)

        hbox = QtWidgets.QHBoxLayout()
        okButton = QtWidgets.QPushButton("Ok")
        okButton.clicked.connect(self.accept)
        cancelButton = QtWidgets.QPushButton("Cancel")
        cancelButton.clicked.connect(self.reject)
        hbox.addStretch(1)
        hbox.addWidget(okButton)
        hbox.addWidget(cancelButton)

        vbox.addLayout(hbox)
        self.setLayout(vbox)
        self.resize(500, 100)

    def textValue(self):
        return self._edit.text()

    def setTextValue(self, text):
        self._edit.setText(text)

    def setLabelText(self, text):
        self._label.setText(text)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    values = ['(A)', '(B)', '(C)', '@home', '@call', '@work', '+qtodotxt', '+sqlvisualizer']
    view = TaskEditorDialog(values)
    view.show()
    sys.exit(app.exec_())



from setuptools import setup, find_packages

import sys


setup(name="qtodotxt", 
      version="1.9.0",
      description="Cross Platform todo.txt GUI",
      author="QTT Development Team",
      author_email="qtodotxt@googlegroups.com",
      url='https://github.com/QTodoTxt/QTodoTxt',
      packages=find_packages(include=("*.png")) + ["qtodotxt.ui.resources", "qtodotxt.ui.resources.css"],
      package_data={"": ["*.png", "*.svg", "*.css", "*.ico"]},
      provides=["qtodotxt"],
      depends=["python-dateutil"],
      license="GNU General Public License v3 or later",
      #install_requires=install_requires,
      classifiers=["Environment :: X11 Applications :: Qt",
                   "Programming Language :: Python :: 3",
                   "Development Status :: 4 - Beta",
                   "Intended Audience :: End Users/Desktop",
                   "Operating System :: OS Independent",
                   "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)"
                   ],
      entry_points={'console_scripts': 
                    [
                        'qtodotxt = qtodotxt.app:run'
                    ]
                    }
      )




from PyQt5 import QtWidgets, QtCore


class Dialogs(object):

    def __init__(self, parent_window, default_title):
        self._parent_window = parent_window
        self._default_title = default_title

    def showMessage(self, message, title=None):
        if not title:
            title = self._default_title
        QtWidgets.QMessageBox.information(self._parent_window, title, message)

    def showError(self, message, title=None):
        if not title:
            title = self._default_title + ' - Error'
        QtWidgets.QMessageBox.critical(self._parent_window, title, message)

    def showSaveDiscardCancel(self, message):
        """
        Returns:
            QtWidgets.QMessageBox.Save or
            QtWidgets.QMessageBox.Discard or
            QtWidgets.QMessageBox.Cancel
        """
        dialog = QtWidgets.QMessageBox(self._parent_window)
        dialog.setWindowTitle('%s - Confirm' % self._default_title)
        dialog.setText(message)
        dialog.setStandardButtons(
            QtWidgets.QMessageBox.Save |
            QtWidgets.QMessageBox.Discard |
            QtWidgets.QMessageBox.Cancel)
        return dialog.exec_()

    def showConfirm(self, message):
        result = QtWidgets.QMessageBox.question(
            self._parent_window,
            '%s - Confirm' % self._default_title,
            message,
            buttons=QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
            defaultButton=QtWidgets.QMessageBox.Yes)
        return result == QtWidgets.QMessageBox.Yes


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    _tr = QtCore.QCoreApplication.translate
    service = Dialogs(None, 'Default Title')
    service.showMessage("DialogsService.message()")
    service.showError("DialogsService.error()")
    result = service.showSaveDiscardCancel(_tr("misc_dialog", "Unsaved changes..."))
    message = _tr("misc_dialog", 'You clicked ')
    if result == QtWidgets.QMessageBox.Save:
        message += _tr("misc_dialog", '"Save"')
    elif result == QtWidgets.QMessageBox.Discard:
        message += _tr("misc_dialog", '"Discard"')
    else:
        message += _tr("misc_dialog", '"Cancel"')
    service.showMessage(message)
    service.showConfirm(_tr("misc_dialog", 'Sure?'))



from PyQt5 import QtWidgets


__version__ = "1.7.0"

description = """<p>QTodoTxt is a cross-platform UI client for todo.txt files
 (see <a href="http://todotxt.com">http://todotxt.com</a>)</p>

Copyright &copy; David Elentok 2011<br/>
Copyright &copy; Matthieu Nantern 2013-2015<br/>
Copyright &copy; QTT Development Team 2015-2016

<h2>Links</h2>
<ul>
<li>Project Page: <a href="https://github.com/QTodoTxt/QTodoTxt">https://github.com/QTodoTxt/QTodoTxt</a></li>
</ul>

<h2>Credits</h2>

<ul>
    <li>Concept by <a href="http://ginatrapani.org/">Gina Trapani</a></li>
    <li>Icons by <a href="http://tango.freedesktop.org/">The Tango! Desktop Project</a>
        and <a href="http://www.famfamfam.com/lab/icons/silk/">Mark James</a></li>
    <li>Original code by <a href="http://elentok.blogspot.com">David Elentok</a></li>
</ul>

<h2>License</h2>

<p>This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.</p>

<p>This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.</p>

<p>You should have received a copy of the GNU General Public License
along with this program.  If not, see
<a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.</p>
"""


def _getAboutText():
    parts = ["<h1>About QTodoTxt %s</h1>\n" % __version__, description]
    return ''.join(parts)


def show(parent=None):
    text = _getAboutText()
    QtWidgets.QMessageBox.information(parent, 'About QTodoTxt', text)



from datetime import datetime, date, time
import re

from PyQt5 import QtCore
from enum import Enum
from qtodotxt.lib.task_htmlizer import TaskHtmlizer


class recursiveMode(Enum):
    completitionDate = 0  # Original due date mode: Task recurs from original due date
    originalDueDate = 1  # Completion date mode: Task recurs from completion date


class recursion:
    mode = None
    increment = None
    interval = None

    def __init__(self, arg_mode, arg_increment, arg_interval):
        self.mode = arg_mode
        self.increment = arg_increment
        self.interval = arg_interval


class Task(QtCore.QObject):
    """
    Represent a task as defined in todo.txt format
    Take a line in todo.txt format as argument
    Arguments are read-only, reparse string to modify them or
    use one the modification methods such as setCompleted()
    """

    modified = QtCore.pyqtSignal(object)

    def __init__(self, line):
        QtCore.QObject.__init__(self)
        settings = QtCore.QSettings()
        self._highest_priority = 'A'
        self._lowest_priority = settings.value("lowest_priority", "D")

        # all other class attributes are defined in _reset method
        # which is called in _parse
        self._parse(line)

    def __str__(self):
        return self._text

    def __repr__(self):
        return "Task({})".format(self._text)

    def _reset(self):
        self.contexts = []
        self.projects = []
        self.priority = ""
        self.is_complete = False
        self.completion_date = None
        self.creation_date = None
        self.is_future = False
        self.threshold_error = ""
        self._text = ''
        self.description = ''
        self.due = None
        self.due_error = ""
        self.threshold = None
        self.keywords = {}
        self.recursion = None

    def _parse(self, line):
        """
        parse a task formated as string in todo.txt format
        """
        self._reset()
        words = line.split(' ')
        if words[0] == "x":
            self.is_complete = True
            words = words[1:]
            # parse next word as a completion date
            # required by todotxt but often not here
            self.completion_date = self._parseDate(words[0])
            if self.completion_date:
                words = words[1:]
        elif re.search(r'^\([A-Z]\)$', words[0]):
            self.priority = words[0][1:-1]
            words = words[1:]

        dato = self._parseDate(words[0])
        if dato:
            self.creation_date = dato
            words = words[1:]

        self.description = " ".join(words)
        for word in words:
            self._parseWord(word)
        self._text = line

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, txt):
        self._parse(txt)
        self.modified.emit(self)

    def _parseWord(self, word):
        if len(word) > 1:
            if word.startswith('@'):
                self.contexts.append(word[1:])
            elif word.startswith('+'):
                self.projects.append(word[1:])
            elif ":" in word:
                key, val = word.split(":", 1)
                self.keywords[key] = val
                if word.startswith('due:'):
                    self.due = self._parseDateTime(word[4:])
                    if not self.due:
                        print("Error parsing due date '{}'".format(word))
                        self.due_error = word[4:]
                elif word.startswith('t:'):
                    self.threshold = self._parseDateTime(word[2:])
                    if not self.threshold:
                        print("Error parsing threshold '{}'".format(word))
                        self.threshold_error = word[2:]
                    else:
                        if self.threshold > datetime.today():
                            self.is_future = True
                elif word.startswith('rec:'):
                    self._parseRecurrence(word)

    def _parseRecurrence(self, word):
        # Original due date mode
        if word[4] == '+':
            # Test if chracters have the right format
            if re.match('^[1-9][bdwmy]', word[5:7]):
                self.recursion = recursion(recursiveMode.originalDueDate, word[5], word[6])
            else:
                print("Error parsing recurrence '{}'".format(word))
        # Completion mode
        else:
            # Test if chracters have the right format
            if re.match('^[1-9][bdwmy]', word[4:6]):
                self.recursion = recursion(recursiveMode.completitionDate, word[4], word[5])
            else:
                print("Error parsing recurrence '{}'".format(word))

    def _parseDate(self, string):
        try:
            return datetime.strptime(string, '%Y-%m-%d').date()
        except ValueError:
            return None

    def _parseDateTime(self, string):
        try:
            return datetime.strptime(string, '%Y-%m-%d')
        except ValueError:
            try:
                return datetime.strptime(string, '%Y-%m-%dT%H:%M')
            except ValueError:
                return None

    @property
    def dueString(self):
        return dateString(self.due)

    @staticmethod
    def updateDateInTask(text, newDate):
        # FIXME: This method has nothing to do in this class, move womewhere else
        # (A) 2016-12-08 Feed Schrodinger's Cat rec:9w due:2016-11-23
        text = re.sub('\sdue\:[0-9]{4}\-[0-9]{2}\-[0-9]{2}', ' due:' + str(newDate)[0:10], text)
        return text

    @property
    def thresholdString(self):
        return dateString(self.threshold)

    def setCompleted(self):
        """
        Set a task as completed by inserting a x and current date at the begynning of line
        """
        if self.is_complete:
            return
        self.completion_date = date.today()
        date_string = self.completion_date.strftime('%Y-%m-%d')
        self._text = 'x %s %s' % (date_string, self._text)
        self.is_complete = True
        self.modified.emit(self)

    def setPending(self):
        """
        Unset completed flag from task
        """
        if not self.is_complete:
            return
        words = self._text.split(" ")
        d = self._parseDate(words[1])
        if d:
            self._text = " ".join(words[2:])
        else:
            self._text = " ".join(words[1:])
        self.is_complete = False
        self.completion_date = None
        self.modified.emit(self)

    def toHtml(self):
        """
        return a task as an html block which is a pretty display of a line in todo.txt format
        """
        htmlizer = TaskHtmlizer()
        return htmlizer.task2html(self)

    def increasePriority(self):
        if self.is_complete:
            return
        if not self.priority:
            self.priority = self._lowest_priority
            self._text = "({}) {}".format(self.priority, self._text)
        elif self.priority != self._highest_priority:
            self.priority = chr(ord(self.priority) - 1)
            self._text = "({}) {}".format(self.priority, self._text[4:])
        self.modified.emit(self)

    def decreasePriority(self):
        if self.is_complete:
            return
        if self.priority >= self._lowest_priority:
            self.priority = ""
            self._text = self._text[4:]
            self._text = self._text.replace("({})".format(self.priority), "", 1)
        elif self.priority:
            oldpriority = self.priority
            self.priority = chr(ord(self.priority) + 1)
            self._text = self._text.replace("({})".format(oldpriority), "({})".format(self.priority), 1)
        self.modified.emit(self)

    def __eq__(self, other):
        return self._text == other.text

    def __lt__(self, other):
        if self.is_complete != other.is_complete:
            return self._lowerCompleteness(other)
        if self.priority != other.priority:
            if not self.priority:
                return True
            if not other.priority:
                return False
            return self.priority > other.priority
        # order the other tasks alphabetically
        return self._text > other.text

    def _lowerCompleteness(self, other):
        if self.is_complete and not other.is_complete:
            return True
        if not self.is_complete and other.is_complete:
            return False
        raise RuntimeError("Could not compare completeness of 2 tasks, please report")


def dateString(date):
    """
    Return a datetime as a nicely formatted string
    """
    if date.time() == time.min:
        return date.strftime('%Y-%m-%d')
    else:
        return date.strftime('%Y-%m-%d %H:%M')


def filterTasks(filters, tasks):
    if None in filters:  # FIXME: why??? if a filter is None we can just ignore it
        return tasks

    filteredTasks = []
    for task in tasks:
        for myfilter in filters:
            if myfilter.isMatch(task):
                filteredTasks.append(task)
                break
    return filteredTasks



from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets
from qtodotxt.lib.filters import ContextFilter, CompleteTasksFilter, DueFilter, DueOverdueFilter, DueThisMonthFilter, \
    DueThisWeekFilter, DueTodayFilter, DueTomorrowFilter, HasContextsFilter, HasDueDateFilter, HasProjectsFilter, \
    ProjectFilter, UncategorizedTasksFilter, AllTasksFilter, PriorityFilter, HasPriorityFilter


class FiltersTreeView(QtWidgets.QWidget):

    filterSelectionChanged = QtCore.pyqtSignal(list)

    def __init__(self, parent=None):
        super(FiltersTreeView, self).__init__(parent)
        self.style = ":/white_icons"
        if str(QtCore.QSettings().value("color_schem", "")).find("dark") >= 0:
            self.style = ":/dark_icons"
        self._filterItemByFilterType = dict()
        self._filterIconByFilterType = dict()
        self._treeItemByFilterType = dict()
        self._initUI()

    def getSelectedFilters(self):
        items = self._tree.selectedItems()
        filters = [item.filter for item in items]
        return filters

    def getSelectedFilterNames(self):
        return [f.text for f in self.getSelectedFilters()]

    def setSelectedFiltersByNames(self, names):
        # FIXME: seems selecting an item deselect previous item....
        # we could also support reselecting dynamic filters....
        if not isinstance(names, (list, tuple)):
            names = [names]
        for name in names:
            if name == "All":
                self.selectAllTasksFilter()
            elif name == "Incomplete":
                self.selectIncompleteTasksFilter()
            elif name == "Complete":
                self._completeTasksItem.setSelected(True)
            elif name == "Uncategorized":
                self._uncategorizedTasksItem.setSelected(True)

    def clear(self):
        self._tree.clear()
        self._addDefaultTreeItems(self._tree)
        self._initFilterTypeMappings()

    def clearSelection(self):
        self._tree.clearSelection()

    def addFilter(self, filter, number=0):
        parentItem = self._filterItemByFilterType[type(filter)]
        icon = self._filterIconByFilterType[type(filter)]
        FilterTreeWidgetItem(parentItem, ["%s (%d)" % (filter.text, number)], filter=filter, icon=icon)
        parentItem.setExpanded(True)
        parentItem.sortChildren(0, QtCore.Qt.AscendingOrder)

    # Predefined sorting for due ranges
    def addDueRangeFilter(self, filter, number=0, sortKey=0):
        parentItem = self._dueItem
        icon = self._filterIconByFilterType[type(filter)]
        FilterTreeWidgetItem(parentItem, ["%s (%d)" % (filter.text, number)], filter=filter, icon=icon, order=sortKey)
        parentItem.setExpanded(True)
        parentItem.sortChildren(1, QtCore.Qt.AscendingOrder)

    def updateTopLevelTitles(self, counters, show_completed=False):
        nbPending = counters['Pending']
        nbDue = counters['Due']
        nbUncategorized = counters['Uncategorized']
        nbContexts = counters['Contexts']
        nbProjects = counters['Projects']
        nbComplete = counters['Complete']
        nbContCompl = counters['ContCompl']
        nbProjCompl = counters['ProjCompl']
        nbDueCompl = counters['DueCompl']
        nbUncatCompl = counters['UncatCompl']
        nbPriority = counters['Priority']
        nbPrioCompl = counters['PrioCompl']

        self._completeTasksItem.setText(0, "Complete (%d)" % nbComplete)
        if (show_completed is True):
            self._allTasksItem.setText(0, "All ({0}; {1})".format(nbPending, nbComplete))
            self._dueItem.setText(0, "Due ({0}; {1})".format(nbDue, nbDueCompl))
            self._contextsItem.setText(0, "Contexts ({0}; {1})".format(nbContexts, nbContCompl))
            self._projectsItem.setText(0, "Projects ({0}; {1})".format(nbProjects, nbProjCompl))
            self._priorityItem.setText(0, "Priority ({0}; {1})".format(nbPriority, nbPrioCompl))
            self._uncategorizedTasksItem.setText(0, "Uncategorized ({0}; {1})".format(nbUncategorized, nbUncatCompl))
        else:
            self._allTasksItem.setText(0, "All (%d)" % nbPending)
            self._contextsItem.setText(0, "Contexts (%d)" % nbContexts)
            self._projectsItem.setText(0, "Projects (%d)" % nbProjects)
            self._dueItem.setText(0, "Due (%d)" % nbDue)
            self._priorityItem.setText(0, "Priority (%d)" % nbPriority)
            self._uncategorizedTasksItem.setText(0, "Uncategorized (%d)" % nbUncategorized)

    def selectAllTasksFilter(self):
        self._allTasksItem.setSelected(True)

    def _selectItem(self, item):
        if item:
            item.setSelected(True)
            self._tree.setCurrentItem(item)

    def _selectContext(self, context):
        item = self._findItem(context, self._contextsItem)
        self._selectItem(item)

    def _selectProject(self, project):
        item = self._findItem(project, self._projectsItem)
        self._selectItem(item)

    def _selectDueRange(self, due):
        item = self._findItem(due, self._dueItem)
        self._selectItem(item)

    def _selectPriority(self, priority):
        item = self._findItem(priority, self._priorityItem)
        self._selectItem(item)

    def _findItem(self, text, parentItem):
        for index in range(parentItem.childCount()):
            child = parentItem.child(index)
            # Remove counter on the tree: context (3) for example
            childText = child.text(0).rpartition(' ')[0]
            if childText == text:
                return child
        return None

    def _initUI(self):
        layout = QtWidgets.QGridLayout()
        self.setLayout(layout)
        offset = QtCore.QSettings().value("ui_margin_offset", -4)
        self.setContentsMargins(offset, offset, 2 * offset, offset)
        self._tree = self._createTreeWidget()
        layout.addWidget(self._tree)

    def _createTreeWidget(self):
        tree = QtWidgets.QTreeWidget()
        tree.header().hide()
        tree.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        tree.itemSelectionChanged.connect(self._tree_itemSelectionChanged)
        self._addDefaultTreeItems(tree)
        self._initFilterTypeMappings()
        return tree

    def _addDefaultTreeItems(self, tree):
        self._allTasksItem = FilterTreeWidgetItem(None, ['All'],
                                                  AllTasksFilter(),
                                                  QtGui.QIcon(self.style + '/resources/FilterAll.png'))
        self._dueItem = FilterTreeWidgetItem(None, ['Due'],
                                             HasDueDateFilter(), QtGui.QIcon(self.style + '/resources/FilterDue.png'))
        self._uncategorizedTasksItem = FilterTreeWidgetItem(
            None, ['Uncategorized'],
            UncategorizedTasksFilter(), QtGui.QIcon(self.style + '/resources/FilterUncategorized.png'))
        self._contextsItem = FilterTreeWidgetItem(None, ['Contexts'],
                                                  HasContextsFilter(),
                                                  QtGui.QIcon(self.style + '/resources/FilterContexts.png'))
        self._projectsItem = FilterTreeWidgetItem(None, ['Projects'],
                                                  HasProjectsFilter(),
                                                  QtGui.QIcon(self.style + '/resources/FilterProjects.png'))
        self._priorityItem = FilterTreeWidgetItem(None, ['Priorities'],
                                                  HasPriorityFilter(),
                                                  QtGui.QIcon(self.style + '/resources/FilterComplete.png'))
        self._completeTasksItem = FilterTreeWidgetItem(None, ['Complete'],
                                                       CompleteTasksFilter(),
                                                       QtGui.QIcon(self.style + '/resources/FilterComplete.png'))
        tree.addTopLevelItems([
            self._allTasksItem, self._uncategorizedTasksItem, self._dueItem, self._contextsItem, self._projectsItem,
            self._priorityItem, self._completeTasksItem
        ])

    def _initFilterTypeMappings(self):
        self._filterItemByFilterType[ContextFilter] = self._contextsItem
        self._filterItemByFilterType[ProjectFilter] = self._projectsItem
        self._filterItemByFilterType[DueFilter] = self._dueItem
        self._filterItemByFilterType[PriorityFilter] = self._priorityItem

        self._filterIconByFilterType[ContextFilter] = QtGui.QIcon(self.style + '/resources/FilterContexts.png')
        self._filterIconByFilterType[ProjectFilter] = QtGui.QIcon(self.style + '/resources/FilterProjects.png')

        self._filterIconByFilterType[DueTodayFilter] = QtGui.QIcon(self.style + '/resources/FilterDueToday.png')
        self._filterIconByFilterType[DueTomorrowFilter] = QtGui.QIcon(self.style + '/resources/FilterDueTomorrow.png')
        self._filterIconByFilterType[DueThisWeekFilter] = QtGui.QIcon(self.style + '/resources/FilterDueWeek.png')
        self._filterIconByFilterType[DueThisMonthFilter] = QtGui.QIcon(self.style + '/resources/FilterDueMonth.png')
        self._filterIconByFilterType[DueOverdueFilter] = QtGui.QIcon(self.style + '/resources/FilterDueOverdue.png')
        self._filterIconByFilterType[PriorityFilter] = QtGui.QIcon(self.style + '/resources/FilterComplete.png')

        self._treeItemByFilterType[AllTasksFilter] = self._allTasksItem
        self._treeItemByFilterType[UncategorizedTasksFilter] = self._uncategorizedTasksItem
        self._treeItemByFilterType[CompleteTasksFilter] = self._completeTasksItem
        self._treeItemByFilterType[HasProjectsFilter] = self._projectsItem
        self._treeItemByFilterType[HasDueDateFilter] = self._dueItem
        self._treeItemByFilterType[HasContextsFilter] = self._contextsItem
        self._treeItemByFilterType[HasPriorityFilter] = self._priorityItem

    def _tree_itemSelectionChanged(self):
        self.filterSelectionChanged.emit(self.getSelectedFilters())

    def selectFilter(self, filter):
        if isinstance(filter, ContextFilter):
            self._selectContext(filter.text)
        elif isinstance(filter, ProjectFilter):
            self._selectProject(filter.text)
        elif isinstance(filter, DueFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, DueTodayFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, DueTomorrowFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, DueThisWeekFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, DueThisMonthFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, DueOverdueFilter):
            self._selectDueRange(filter.text)
        elif isinstance(filter, PriorityFilter):
            self._selectPriority(filter.text)
        else:
            item = self._treeItemByFilterType[type(filter)]
            self._selectItem(item)


class FilterTreeWidgetItem(QtWidgets.QTreeWidgetItem):
    def __init__(self, parent, strings, filter=None, icon=None, order=None):
        QtWidgets.QTreeWidgetItem.__init__(self, parent, strings)
        self.filter = filter
        if order:
            self.setText(1, str(order))
        if icon:
            self.setIcon(0, icon)



import os
import sys
from PyQt5.QtGui import QIcon


def _getRoot():
    root = ''
    if sys.argv[0].lower().endswith('.exe'):
        root = os.path.dirname(sys.argv[0])
    elif getattr(sys, 'frozen', False):
        root = os.environ['RESOURCEPATH']
    else:
        file = None
        try:
            file = __file__
        except NameError:
            file = sys.argv[0]
        root = os.path.dirname(os.path.abspath(file))
    return root


def __getResourcesRoot():
    return os.path.join(_getRoot(), 'resources')


def getResourcePath(resource_name):
    return os.path.join(__getResourcesRoot(), resource_name)


def getIcon(resource_name):
    return QIcon(getResourcePath(resource_name))



import unittest
from PyQt5 import QtCore
from datetime import date

from qtodotxt.lib import tasklib
from qtodotxt.lib.file import File
from qtodotxt.lib.filters import IncompleteTasksFilter, ContextFilter, ProjectFilter, DueThisMonthFilter, \
    DueThisWeekFilter, DueTodayFilter
from qtodotxt.ui.controllers.filters_tree_controller import FiltersTreeController


class FakeTreeView(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self.filters = []
        self.selectedFilters = []

    def addFilter(self, filter, number=0):
        filter.text = "%s (%d)" % (filter.text, number)
        self.filters.append(filter)

    def addDueRangeFilter(self, filter, number=0, sortKey=0):
        filter.text = "%s (%d)" % (filter.text, number)
        self.filters.append(filter)

    def clear(self):
        self.filters = []

    def clearSelection(self):
        self.selectedFilters = []

    def selectFilter(self, filter):
        if filter not in self.selectedFilters:
            self.selectedFilters.append(filter)

    def getSelectedFilters(self):
        return self.selectedFilters

    filterSelectionChanged = QtCore.pyqtSignal()

    def selectAllTasksFilter(self):
        self.selectedFilters = [IncompleteTasksFilter()]

    def updateTopLevelTitles(self, counters, show_completed=False):
        return


class Test(unittest.TestCase):

    def _createMockFile(self):
        today = today = date.today().strftime('%Y-%m-%d')
        file = File()
        file.tasks.append(tasklib.Task('my task1 @context1'))
        file.tasks.append(tasklib.Task('my task2 @context1 @context2'))
        file.tasks.append(tasklib.Task('due:' + today + ' my task3 +project1 @context2'))
        file.tasks.append(tasklib.Task('due:' + today + ' my task4'))
        return file

    def test_showFilters(self):
        # arrange
        view = FakeTreeView()
        controller = FiltersTreeController(view)
        file = self._createMockFile()

        # act
        controller.showFilters(file)

        sortedFilter = sorted(view.filters, key=lambda filter: filter.text)

        # assert
        self.assertEqual(1, len(view.selectedFilters),
                         'There should be only 1 selected filter (actual: %s)' % view.selectedFilters)
        self.assertIsInstance(view.selectedFilters[0], IncompleteTasksFilter,
                              'selected filter #1 should be instance of IncompleteTasksFilter (actual: %s)'
                              % view.selectedFilters[0])

        self.assertEqual(6, len(view.filters), 'There should be 6 filters (actual: %d)' % len(view.filters))

        filter = sortedFilter[0]
        self.assertIsInstance(filter, DueThisMonthFilter,
                              'Filter #1 should be instance of DueThisMonthFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'This month (2)',
                         'Filter #1 text should be "This month" (actual: "%s")' % filter.text)

        filter = sortedFilter[1]
        self.assertIsInstance(filter, DueThisWeekFilter,
                              'Filter #2 should be instance of DueThisWeekFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'This week (2)',
                         'Filter #2 text should be "This week" (actual: "%s")' % filter.text)

        filter = sortedFilter[2]
        self.assertIsInstance(filter, DueTodayFilter,
                              'Filter #3 should be instance of DueTodayFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'Today (2)',
                         'Filter #3 text should be "Today" (actual: "%s")' % filter.text)

        filter = sortedFilter[3]
        self.assertIsInstance(filter, ContextFilter,
                              'Filter #4 should be instance of ContextFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'context1 (2)',
                         'Filter #4 text should be "context1" (actual: "%s")' % filter.text)

        filter = sortedFilter[4]
        self.assertIsInstance(filter, ContextFilter,
                              'Filter #5 should be instance of ContextFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'context2 (2)',
                         'Filter #5 text should be "%s" (actual: context2)' % filter.text)

        filter = sortedFilter[5]
        self.assertIsInstance(filter, ProjectFilter,
                              'Filter #6 should be instance of ProjectFilter (actual: %s)' % str(type(filter)))
        self.assertEqual(filter.text, 'project1 (1)',
                         'Filter #6 text should be "%s" (actual: project1)' % filter.text)

    def test_showFilters_afterAddingNewContext(self):
        # arrange
        view = FakeTreeView()
        controller = FiltersTreeController(view)
        file = self._createMockFile()
        controller.showFilters(file)
        original_filter0 = view.filters[0]
        view.clearSelection()
        view.selectFilter(view.filters[0])
        file.tasks[2] = tasklib.Task(file.tasks[2].text + " @context3")

        # act
        controller.showFilters(file)

        # assert
        self.assertEquals(
            7,
            len(view.filters),
            'There should be 7 filters (actual: %s)' % view.selectedFilters)

        sortedFilter = sorted(view.filters, key=lambda filter: filter.text)
        print(sortedFilter)

        self.assertEquals(
            1, len(view.selectedFilters),
            'There should be 1 selected filters (actual: %s)' % view.selectedFilters)

        # Due filters are not sorted
        filter1_text = sortedFilter[3].text
        self.assertEqual("context1 (2)", filter1_text,
                         'Filter #1 context should be "context1 (2)" (actual: "%s")' % filter1_text)

        expectedselectedfilter = original_filter0.text
        self.assertSequenceEqual(
            expectedselectedfilter,
            view.selectedFilters[0].text,
            'Wrong selected filters (expected: %s, actual: %s)' %
            (expectedselectedfilter, view.selectedFilters))
